<?php
/**
 * Rules helper
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Helper\Rules;

use DDWCWalletManagement\Helper\Error\DDWCWM_Error_Helper;

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'DDWCWM_Rules_Helper' ) ) {
	/**
	 * Rules helper class
	 */
	class DDWCWM_Rules_Helper {

		/**
		 * Error trait
		 * 
		 * @var object
		 */
		use DDWCWM_Error_Helper;

		/**
		 * Database Object
		 *
		 * @var object
		 */
		protected $wpdb;

		/**
		 * Rules table Variable
		 *
		 * @var string
		 */
        protected $rules_table;

		/**
		 * Construct
		 */
		public function __construct() {
			global $wpdb;
            $this->wpdb           = $wpdb;
            $this->rules_table    = $this->wpdb->prefix . 'ddwcwm_cashback_rules';
            $this->postmeta_table = $this->wpdb->prefix . 'postmeta';
		}

        /**
		 * Prepare cashback rule data and save to DB function
		 *
		 * @param array $data
		 * @return int
		 */
		public function ddwcwm_prepare_cashback_rule_data_and_save( $data, $old_cashback_rules, $old_product_rules ) {
			// General Rules args
            $amount_from     = ! empty( $data[ 'ddwcwm_amount_from' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_amount_from' ] ) ) : [];
            $amount_to       = ! empty( $data[ 'ddwcwm_amount_to' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_amount_to' ] ) ) : [];
            $basis           = ! empty( $data[ 'ddwcwm_rule_basis' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_rule_basis' ] ) ) : [];
            $cashback_type   = ! empty( $data[ 'ddwcwm_cashback_type' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_cashback_type' ] ) ) : [];
            $cashback_amount = ! empty( $data[ 'ddwcwm_cashback_amount' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_cashback_amount' ] ) ) : [];
            $rule_status     = ! empty( $data[ 'ddwcwm_rule_status' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_rule_status' ] ) ) : [];

			if ( ! empty( $amount_from ) && ! empty( $amount_to ) && ! empty( $basis ) && ! empty( $cashback_type ) && ! empty( $cashback_amount ) && ! empty( $rule_status ) ) {
				$number_of_cashback_rules = count( $amount_from );

				if ( ( count( $amount_from ) + count( $amount_to ) + count( $basis ) + count( $cashback_type ) + count( $cashback_amount ) + count( $rule_status ) ) === ( $number_of_cashback_rules * 6 ) ) {
					foreach ( $amount_from as $amount_from_id => $amount_from_value ) {
						if ( $amount_from[ $amount_from_id ] > $amount_to[ $amount_from_id ] ) {
							$this->error_helper->set_error_code( 1 );
							$error = 1;
							break;
						}

						if ( ! is_numeric( $cashback_amount[ $amount_from_id ] ) ) {
							$error = 1;
							break;
						}

						foreach ( $amount_from as $key => $amount_from_val ) {
							if ( $basis[ $amount_from_id ] != $basis[ $key ] ) {
								continue;
							}

							if ( $amount_from_id === $key ) {
								continue;
							}

							if ( $amount_from_value == $amount_from[ $key ] ) {
								$error = 1;
								break 2;
							}

							if ( $amount_from[ $amount_from_id ] > $amount_to[ $amount_from_id ] ) {
								$error = 1;
								break 2;
							}

							if ( $amount_from[ $key ] <= $amount_from[ $amount_from_id ] && $amount_from[ $amount_from_id ] <= $amount_to[ $key ] ) {
								$error = 1;
								break 2;
							}

							if ( $amount_from[ $key ] <= $amount_to[ $amount_from_id ] && $amount_to[ $amount_from_id ] <= $amount_to[ $key ] ) {
								$error = 1;
								break 2;
							}
						}
					}
				} else {
					$error = 1;
				}

				if ( empty( $error ) ) {
					if( ! empty( $old_cashback_rules ) ) {
						foreach ( $old_cashback_rules as $key => $old_cashback_rule ) {
							if ( ! array_key_exists( $old_cashback_rule[ 'id' ], $amount_from ) ) {
								$this->ddwcwm_delete_cashback_rule( $old_cashback_rule[ 'id' ] );
							}
						}
					}

					foreach ( $amount_from as $cashback_id => $amount_from_value ) {
						$cashback_rule_data = [
							'amount_from'     => $amount_from[ $cashback_id ],
							'amount_to'       => $amount_to[ $cashback_id ],
							'basis'           => $basis[ $cashback_id ],
							'cashback_type'   => $cashback_type[ $cashback_id ],
							'cashback_amount' => $cashback_amount[ $cashback_id ],
							'status'          => $rule_status[ $cashback_id ],
						];

						$cashback_id = $this->ddwcwm_save_cashback_rule( $cashback_rule_data, $cashback_id );
					}
				} else {
					$this->ddwcwm_print_notification( esc_html__( 'General cashback rules having empty or invalid fields.', 'ddwc-wallet-management' ), 'error' );
				}
			} else {
				if ( ! empty( $old_cashback_rules ) ) {
					foreach ( $old_cashback_rules as $key => $old_cashback_rule ) {
						if ( ! array_key_exists( $old_cashback_rule[ 'id' ], $amount_from ) ) {
							$this->ddwcwm_delete_cashback_rule( $old_cashback_rule[ 'id' ] );
						}
					}
				}
			}

			// Product Rules args
            $rule_products         = ! empty( $data[ 'ddwcwm_rule_products' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_rule_products' ] ) ) : [];
            $product_quantity      = ! empty( $data[ 'ddwcwm_product_quantity' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_product_quantity' ] ) ) : [];
            $product_cashback_type = ! empty( $data[ 'ddwcwm_product_cashback_type' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_product_cashback_type' ] ) ) : [];
            $product_cashback      = ! empty( $data[ 'ddwcwm_product_cashback' ] ) ? array_map( 'sanitize_text_field', wp_unslash( $data[ 'ddwcwm_product_cashback' ] ) ) : [];

			if ( ! empty( $rule_products ) && ! empty( $product_quantity ) && ! empty( $product_cashback_type ) && ! empty( $product_cashback ) ) {
				$number_of_product_cashback_rules = count( $rule_products );

				if ( ( count( $rule_products ) + count( $product_quantity ) + count( $product_cashback_type ) + count( $product_cashback ) ) === ( $number_of_product_cashback_rules * 4 ) ) {
					if ( ! empty( $old_product_rules ) ) {
						foreach ( $old_product_rules as $old_product_rule ) {
							if ( ! in_array( $old_product_rule[ 'post_id' ], $rule_products ) ) {
								delete_post_meta( $old_product_rule[ 'post_id' ], '_ddwcwm_cashback_data' );
							}
						}
					}

					foreach ( $rule_products as $key => $product_id ) {
						$cashback_data = [
							'quantity'        => $product_quantity[ $key ],
							'cashback_type'   => $product_cashback_type[ $key ],
							'cashback_amount' => $product_cashback[ $key ],
						];

						update_post_meta( $product_id, '_ddwcwm_cashback_data', $cashback_data );
					}
				} else {
					$this->ddwcwm_print_notification( esc_html__( 'Product cashback rules having empty or invalid fields.', 'ddwc-wallet-management' ), 'error' );
				}
			} else {
				if ( ! empty( $old_product_rules ) ) {
					foreach ( $old_product_rules as $old_product_rule ) {
						if ( ! in_array( $old_product_rule[ 'post_id' ], $rule_products ) ) {
							delete_post_meta( $old_product_rule[ 'post_id' ], '_ddwcwm_cashback_data' );
						}
					}
				}
			}

			if ( empty( $error ) ) {
				$this->ddwcwm_print_notification( esc_html__( 'Cashback Rules has been saved successfully.', 'ddwc-wallet-management' ), 'success' );

				return true;
			}

			return false;
		}

		/**
		 * Save cashback rule to DB function
		 *
		 * @param array $rules_data
		 * @param array $rule_id
		 * @return int
		 */
		public function ddwcwm_save_cashback_rule( $rules_data, $rule_id ) {
            if ( ! empty( $rule_id ) ) {
				$this->wpdb->update(
					$this->rules_table,
					$rules_data,
					[ 'id' => $rule_id ],
					[ '%s', '%s', '%s', '%s', '%s', '%s' ],
					[ '%d' ]
				);
			} else {
				$this->wpdb->insert(
					$this->rules_table,
					$rules_data,
					[ '%s', '%s', '%s', '%s', '%s', '%s' ]
				);

				$rule_id = $this->wpdb->insert_id;
			}

            return $rule_id;
		}

		/**
		 * Get All Cashback Rules function
		 *
		 * @param int $per_page
		 * @param int $offset
		 * @return array
		 */
		public function ddwcwm_get_all_cashback_rules() {
			$rules = $this->wpdb->get_results( "SELECT * FROM $this->rules_table", ARRAY_A );

			return apply_filters( 'ddwcwm_modify_rules_data', $rules );
		}

		/**
		 * Get All Product Cashback Rules function
		 *
		 * @param int $per_page
		 * @param int $offset
		 * @return array
		 */
		public function ddwcwm_get_all_product_cashback_rules() {
			$product_cashback_rules = $this->wpdb->get_results( $this->wpdb->prepare( "SELECT post_id, meta_value FROM $this->postmeta_table WHERE meta_key=%s", '_ddwcwm_cashback_data' ), ARRAY_A );

			return apply_filters( 'ddwcwm_modify_product_rules_data', $product_cashback_rules );
		}

		/**
		 * Delete Cashback Rule function
		 * 
		 * @param int $rule_id
		 * @return int|bool
		 */
		public function ddwcwm_delete_cashback_rule( $rule_id ) {

			return $this->wpdb->delete(
				$this->rules_table,
				[
					'id' => $rule_id
				],
                [ '%d' ]
			);

		}

		/**
		 * Calculate Cashbacks with cart function
		 *
		 * @return array
		 */
		public function ddwcwm_calculate_cashbacks_with_cart() {

			$wallet_topup_pro = get_page_by_path( 'ddwcwm-wallet-topup' , OBJECT, 'product' );

			$wallet_pro_exists = false;

			$cashbacks = [
				'cart'    => 0,
				'topup'   => 0,
				'product' => 0,
			];

			$cart_subtotal = apply_filters( 'ddwcwm_modify_amount_to_base_currency', floatval( WC()->cart->get_subtotal() ) );

			$cart_contents = WC()->cart->get_cart_contents();

			if ( ! empty( $cart_contents ) ) {
				foreach ( $cart_contents as $key => $cart_content ) {
					$product_id = ! empty( $cart_content[ 'variation_id' ] ) ? $cart_content[ 'variation_id' ] : $cart_content[ 'product_id' ];

					if ( $product_id == $wallet_topup_pro->ID ) {
						$wallet_pro_exists = true;

						$cashback_data = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT cashback_amount, cashback_type FROM $this->rules_table WHERE basis=%s AND status=%s AND %f >= amount_from AND amount_to >= %f", 'topup', 'enabled', $cart_subtotal, $cart_subtotal ), ARRAY_A );

						if ( ! empty( $cashback_data ) ) {
							if ( $cashback_data[ 'cashback_type' ] == 'fixed' ) {
								$cashbacks[ 'topup' ] = $cashback_data[ 'cashback_amount' ];
							} else {
								$cashbacks[ 'topup' ] = $cart_subtotal * $cashback_data[ 'cashback_amount' ] / 100;
							}
						}
					} else {
						$product_cashback_data = get_post_meta( $product_id, '_ddwcwm_cashback_data', true );

						if ( ! empty( $product_cashback_data ) ) {
							if ( $cart_content[ 'quantity' ] >= $product_cashback_data[ 'quantity' ] ) {
								$cashback_type = $product_cashback_data[ 'cashback_type' ];
								$cashback_amount = $product_cashback_data[ 'cashback_amount' ];

								if ( $cashback_type == 'fixed' ) {
									$cashbacks[ 'product' ] += $cashback_amount;
								} else if ( $cashback_type == 'percentage' ) {
									$cashbacks[ 'product' ] += $cart_content[ 'line_total' ] * $cashback_amount / 100;
								}
							}
						}
					}
				}

				if ( ! $wallet_pro_exists ) {
					$cashback_data = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT cashback_amount, cashback_type FROM $this->rules_table WHERE basis=%s AND status=%s AND %f >= amount_from AND amount_to >= %f", 'cart', 'enabled', $cart_subtotal, $cart_subtotal ), ARRAY_A );

					if ( ! empty( $cashback_data ) ) {
						if ( $cashback_data[ 'cashback_type' ] == 'fixed' ) {
							$cashbacks[ 'cart' ] = $cashback_data[ 'cashback_amount' ];
						} else {
							$cashbacks[ 'cart' ] = $cart_subtotal * $cashback_data[ 'cashback_amount' ] / 100;
						}
					}
				}

			}

			return $cashbacks;
		}

		/**
		 * Calculate Cashbacks with order function
		 *
		 * @param int $order_id
		 * @return array
		 */
		public function ddwcwm_calculate_cashbacks_with_order( $order_id ) {
			$wallet_topup_pro = get_page_by_path( 'ddwcwm-wallet-topup' , OBJECT, 'product' );

			$wallet_pro_exists = false;

			$cashbacks = [
				'cart'    => 0,
				'topup'   => 0,
				'product' => 0,
			];

			$order          = new \WC_Order( $order_id );
			$order_subtotal = apply_filters( 'ddwcwm_modify_order_subtotal_for_calculating_cashbacks', $order->get_subtotal(), $order );
			$order_items    = $order->get_items();

			if ( ! empty( $order_items ) ) {
				foreach ( $order_items as $key => $cart_content ) {
					$product_id = ! empty( $cart_content[ 'variation_id' ] ) ? $cart_content[ 'variation_id' ] : $cart_content[ 'product_id' ];

					if ( $product_id == $wallet_topup_pro->ID ) {
						$wallet_pro_exists = true;

						$cashback_data = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT cashback_amount, cashback_type FROM $this->rules_table WHERE basis=%s AND status=%s AND %f >= amount_from AND amount_to >= %f", 'topup', 'enabled', $order_subtotal, $order_subtotal ), ARRAY_A );

						if ( ! empty( $cashback_data ) ) {
							if ( $cashback_data[ 'cashback_type' ] == 'fixed' ) {
								$cashbacks[ 'topup' ] = $cashback_data[ 'cashback_amount' ];
							} else {
								$cashbacks[ 'topup' ] = $order_subtotal * $cashback_data[ 'cashback_amount' ] / 100;
							}
						}
					} else {
						$product_cashback_data = get_post_meta( $product_id, '_ddwcwm_cashback_data', true );

						if ( ! empty( $product_cashback_data ) ) {
							if ( $cart_content[ 'quantity' ] >= $product_cashback_data[ 'quantity' ] ) {
								$cashback_type = $product_cashback_data[ 'cashback_type' ];
								$cashback_amount = $product_cashback_data[ 'cashback_amount' ];

								if ( $cashback_type == 'fixed' ) {
									$cashbacks[ 'product' ] += $cashback_amount;
								} else if ( $cashback_type == 'percentage' ) {
									$cashbacks[ 'product' ] += $cart_content[ 'line_total' ] * $cashback_amount / 100;
								}
							}
						}
					}
				}

				if ( ! $wallet_pro_exists ) {
					$cashback_data = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT cashback_amount, cashback_type FROM $this->rules_table WHERE basis=%s AND status=%s AND %f >= amount_from AND amount_to >= %f", 'cart', 'enabled', $order_subtotal, $order_subtotal ), ARRAY_A );

					if ( ! empty( $cashback_data ) ) {
						if ( $cashback_data[ 'cashback_type' ] == 'fixed' ) {
							$cashbacks[ 'cart' ] = $cashback_data[ 'cashback_amount' ];
						} else {
							$cashbacks[ 'cart' ] = $order_subtotal * $cashback_data[ 'cashback_amount' ] / 100;
						}
					}
				}
			}

			return $cashbacks;
		}
	}
}
