<?php
/**
 * Transactions helper
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Helper\Transactions;

use DDWCWalletManagement\Templates\Admin\Transactions\DDWCWM_Transaction_View_Template;
use DDWCWalletManagement\Helper\Error\DDWCWM_Error_Helper;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Transactions_Helper' ) ) {
	/**
	 * Transactions helper class
	 */
	class DDWCWM_Transactions_Helper {
		/**
		 * Database Object
		 *
		 * @var object
		 */
        protected $wpdb;

		/**
		 * Transactions table Variable
		 *
		 * @var string
		 */
		protected $transaction_table;

        /**
		 * Users table Variable
		 *
		 * @var string
		 */
        protected $users_table;

		/**
		 * Construct
		 */
		public function __construct() {
			global $wpdb;
            $this->wpdb = $wpdb;

            $this->transaction_table = $this->wpdb->prefix . 'ddwcwm_transactions';
			$this->users_table = $this->wpdb->users;
		}

		/**
		 * Save transaction to DB function
		 *
		 * @param array $data
		 * @return int
		 */
		public function ddwcwm_save_transaction( $data ) {
			$default_data = [
                'order_id'  => NULL,
				'reference' => '',
				'sender_id' => 0,
                'user_id'   => get_current_user_id(),
                'amount'    => 0,
                'type'      => '',
                'date'      => current_time( 'Y-m-d H:i:s' ),
                'note'      => '',
			];

			$data = wp_parse_args( $data, $default_data );
            $this->wpdb->insert(
                $this->transaction_table,
                $data,
                [ '%d', '%s', '%d', '%d', '%s', '%s', '%s', '%s' ]
            );

            return $this->wpdb->insert_id;
		}

		/**
		 * Get transaction template function
		 *
		 * @return $insert_id
		 */
		public function ddwcwm_get_transaction_template( $id ) {
			$transaction = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT * FROM $this->transaction_table WHERE id=%d", $id ), ARRAY_A );

			if ( ! empty( $transaction ) ) {
				new DDWCWM_Transaction_View_Template( $transaction );
			} else {
				$error_helper = new DDWCWM_Error_Helper();
				$error_helper->ddwcwm_print_notification( esc_html__( 'Invalid Transaction ID.', 'ddwc-wallet-management' ), 'error' );
			}
		}

		/**
		 * Get All transactions function
		 *
		 * @param int $per_page
		 * @param int $offset
		 * @return array
		 */
		public function ddwcwm_get_all_transactions( $per_page, $offset ) {

            $conditions = '';

			if ( ! empty( $_GET[ 'ddwcwm-transaction-type' ] ) ) {
				$conditions .= $this->wpdb->prepare( " AND transactions.type=%s", $_GET[ 'ddwcwm-transaction-type' ] );
			}
			if ( ! empty( $_GET[ 'ddwcwm-transaction-from-date' ] ) ) {
				$transaction_from_date = $_GET[ 'ddwcwm-transaction-from-date' ];
				$transaction_to_date = ! empty( $_GET[ 'ddwcwm-transaction-to-date' ] ) ? $_GET[ 'ddwcwm-transaction-to-date' ] : $transaction_from_date;
				$conditions .= $this->wpdb->prepare( " AND DATE(date) BETWEEN %s AND %s", $transaction_from_date, $transaction_to_date );
			}
			if ( ! empty( $_GET[ 'customer-id' ] ) ) {
				$conditions .= $this->wpdb->prepare( " AND transactions.user_id=%d", $_GET[ 'customer-id' ] );
			}
			if ( ! empty( $_GET[ 's' ] ) ) {
				$conditions .= $this->wpdb->prepare( " AND transactions.id LIKE %s", '%' . $_GET[ 's' ] . '%' );
			}

			$transactions = $this->wpdb->get_results( "SELECT DISTINCT transactions.* FROM $this->transaction_table as transactions LEFT JOIN $this->users_table as users ON transactions.user_id=users.ID WHERE 1=1 $conditions GROUP BY transactions.id ORDER BY transactions.id DESC LIMIT $per_page OFFSET $offset", ARRAY_A );

			return apply_filters( 'ddwcwm_modify_transactions_data', $transactions, $per_page, $offset );

		}

		/**
		 * Get All transactions count function
		 *
		 * @return int
		 */
		public function ddwcwm_get_all_transactions_count() {

            $conditions = '';

			if ( ! empty( $_GET[ 'ddwcwm-transaction-type' ] ) ) {
				$conditions .= $this->wpdb->prepare( " AND transactions.type=%s", $_GET[ 'ddwcwm-transaction-type' ] );
			}
			if ( ! empty( $_GET[ 'ddwcwm-transaction-from-date' ] ) ) {
				$transaction_from_date = $_GET[ 'ddwcwm-transaction-from-date' ];
				$transaction_to_date = ! empty( $_GET[ 'ddwcwm-transaction-to-date' ] ) ? $_GET[ 'ddwcwm-transaction-to-date' ] : $transaction_from_date;
				$conditions .= $this->wpdb->prepare( " AND DATE(date) BETWEEN %s AND %s", $transaction_from_date, $transaction_to_date );
			}
			if ( ! empty( $_GET[ 'customer-id' ] ) ) {
				$conditions .= $this->wpdb->prepare( " AND transactions.user_id=%d", $_GET[ 'customer-id' ] );
			}
			if ( ! empty( $_GET[ 's' ] ) ) {
				$conditions .= $this->wpdb->prepare( " AND transactions.id LIKE %s", '%' . $_GET[ 's' ] . '%' );
			}

			$total_transactions_count = $this->wpdb->get_var( "SELECT count(DISTINCT transactions.id) FROM $this->transaction_table as transactions JOIN $this->users_table as users ON transactions.user_id=users.ID WHERE 1=1 $conditions" );

			return apply_filters( 'ddwcwm_modify_total_transactions_count', $total_transactions_count );

		}

		/**
		 * Return translations for static words function
		 *
		 * @param string $static_word
		 * @return string
		 */
		public static function ddwcwm_get_transactions_translation( $static_word ) {
            $transaction_translation = [
				'credit'                 => esc_html__( 'Credit', 'ddwc-wallet-management' ),
				'debit'                  => esc_html__( 'Debit', 'ddwc-wallet-management' ),
				'manual_debit'           => esc_html__( 'Manual Debit', 'ddwc-wallet-management' ),
				'manual_credit'          => esc_html__( 'Manual Credit', 'ddwc-wallet-management' ),
				'order_credit'           => esc_html__( 'Order Credit', 'ddwc-wallet-management' ),
				'order_debit'            => esc_html__( 'Order Debit', 'ddwc-wallet-management' ),
				'order_cancelled_credit' => esc_html__( 'Order Cancelled Credit', 'ddwc-wallet-management' ),
				'order_cancelled_debit'  => esc_html__( 'Order Cancelled Debit', 'ddwc-wallet-management' ),
				'order_refund'           => esc_html__( 'Order Refund', 'ddwc-wallet-management' ),
				'wallet_topup'           => esc_html__( 'Wallet Topup', 'ddwc-wallet-management' ),
				'cart_cashback'          => esc_html__( 'Cart Cashback', 'ddwc-wallet-management' ),
				'product_cashback'       => esc_html__( 'Product Cashback', 'ddwc-wallet-management' ),
				'topup_cashback'         => esc_html__( 'Topup Cashback', 'ddwc-wallet-management' ),
				'transfer'               => esc_html__( 'Transfer', 'ddwc-wallet-management' ),
				'withdraw'               => esc_html__( 'Withdrawal', 'ddwc-wallet-management' ),
				'wallet_transfer'        => esc_html__( 'Wallet Transfer', 'ddwc-wallet-management' ),
				'withdraw_requested'     => esc_html__( 'Withdraw Requested', 'ddwc-wallet-management' ),
				'withdraw_charges'       => esc_html__( 'Withdraw Charges', 'ddwc-wallet-management' ),
				'withdraw_completed'     => esc_html__( 'Withdraw Completed', 'ddwc-wallet-management' ),
				'withdraw_cancelled'     => esc_html__( 'Withdraw Cancelled', 'ddwc-wallet-management' ),
				'registration_credit'    => esc_html__( 'Registration Credit', 'ddwc-wallet-management' ),
				'referral_credit'        => esc_html__( 'Referral Credit', 'ddwc-wallet-management' ),
			];

			$transaction_translation = apply_filters( 'ddwcwm_modify_transaction_translations', $transaction_translation );

			$result_translation = ! empty( $transaction_translation[ $static_word ] ) ? $transaction_translation[ $static_word ] : $static_word;

			return apply_filters( 'ddwcwm_modify_result_transaction_translation', $result_translation );
		}

		/**
		 * Delete Transaction function
		 * 
		 * @param int $transaction_id
		 * @return int|bool
		 */
		public function ddwcwm_delete_transaction( $transaction_id ) {
			return $this->wpdb->delete(
				$this->transaction_table,
				[
					'id' => $transaction_id
				],
                [ '%d' ]
            );
		}
	}
}
