<?php
/**
 * Admin ajax functions class
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes\Admin;

use DDWCWalletManagement\Helper\Transactions\DDWCWM_Transactions_Helper;
use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;
use DDWCWalletManagement\Includes\Admin\Importer\DDWCWM_Users_Importer_Controller;
use DDWCWalletManagement\Includes\Admin\Exporter\DDWCWM_Users_CSV_Exporter;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Admin_Ajax_Functions' ) ) {
	/**
	 * Admin Ajax fuctions class
	 */
	class DDWCWM_Admin_Ajax_Functions {

		/**
		 * Get Users
		 *
		 * @return void
		 */
		public function ddwcwm_get_users_list() {
			$response = [];
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
				$query = isset( $_POST[ 'query' ] ) ? sanitize_text_field( wp_unslash( $_POST[ 'query' ] ) ) : ''; // wpcs: input var okay.

				$query = new \WP_User_Query( [
                    'search'         => '*' . esc_attr( $query ) . '*',
                    'search_columns' => [ 'user_login', 'user_email' ],
                    'fields'         => [ 'user_email', 'user_login', 'ID' ],
				] );

				$response = [
					'error'   => false,
					'message' => '',
					'users'   => $query->get_results(),
				];
			} else {
				$response = [
					'error'   => true,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}
			wp_send_json( $response );
		}

		/**
		 * Get Users
		 *
		 * @return void
		 */
		public function ddwcwm_get_products_list() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
                // we will pass post IDs and titles to this array.
				$products = [];

				$wallet_topup_pro = get_page_by_path( 'ddwcwm-wallet-topup' , OBJECT, 'product' );

                // you can use WP_Query, query_posts() or get_posts() here - it doesn't matter.
                $search_results = new \WP_Query( [
                    's'                   => esc_attr( $_POST[ 'query' ] ),
                    'post_type'           => [ 'product', 'product_variation' ],
                    'post_status'         => 'publish',
                    'ignore_sticky_posts' => 1,
                    'posts_per_page'      => 10,
                    'search_columns'      => [ 'post_title' ],
                    'fields'              => [ 'ID', 'post_title' ],
                    'post__not_in'        => [ $wallet_topup_pro->ID ],
                ] );

                if ( $search_results->have_posts() ) {
                    while ( $search_results->have_posts() ) {
                        $search_results->the_post();

                        $title = ( mb_strlen( $search_results->post->post_title ) > 50 ) ? mb_substr( $search_results->post->post_title, 0, 49 ) . '...' : $search_results->post->post_title;

                        $products[] = [
                            'ID'    => $search_results->post->ID,
                            'title' => $title,
                        ];
                    }
                }

                wp_send_json( $products );
                die;
            }
		}

		/**
		 * Verify License function
		 *
		 * @return void
		 */
		public function ddwcwm_verify_license() {

			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {

				$purchase_code = ! empty( $_POST[ 'purchase_code' ] ) ? $_POST[ 'purchase_code' ] : '';
				$status        = ! empty( $_POST[ 'status' ] ) ? $_POST[ 'status' ] : '';

				$args = [
					'purchase_code'    => $purchase_code,
					'action'           => $status,
					'user_agent'       => site_url(),
					'multisite'        => is_multisite(),
					'network_site_url' => network_site_url(),
				];

				$response = wp_remote_post( 'https://devdiggers.com/wp-json/ddelv/v1/verify-purchase', [
					'body' => json_encode( apply_filters( 'ddwcwm_modify_verify_license_args', $args ) )
				] );

				$response = json_decode( wp_remote_retrieve_body( $response ) );

				if ( $response->success ) {

					update_option( '_ddwcwm_purchase_code', $purchase_code );

					if ( $status == 'activate' ) {
						update_option( '_ddwcwm_license_activated', true );
					} else {
						delete_option( '_ddwcwm_license_activated' );
					}

				}

				wp_send_json( $response );

			}

		}

		/**
		 * Withdraw request action function
		 *
		 * @return void
		 */
		public function ddwcwm_withdraw_request_action() {
			if ( check_ajax_referer( 'ddwcwm-nonce', 'ddwcwm_nonce', false ) ) {
                $user_id             = ! empty( $_POST[ 'user_id' ] ) ? intval( sanitize_text_field( $_POST[ 'user_id' ] ) ) : '';
                $withdraw_request_id = isset( $_POST[ 'withdraw_request_id' ] ) ? intval( sanitize_text_field( $_POST[ 'withdraw_request_id' ] ) ) : '';
                $perform             = ! empty( $_POST[ 'perform' ] ) ? sanitize_text_field( $_POST[ 'perform' ] ) : '';
                $user_helper         = new DDWCWM_Users_Helper();
                $withdraw_requests   = $user_helper->ddwcwm_get_user_withdraw_requests( $user_id );

				if ( ! empty( $withdraw_requests ) ) {
					$transaction_helper = new DDWCWM_Transactions_Helper();

					if ( 'cancel' === $perform ) {
						$current_user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );

						$amount_before_charges = $withdraw_requests[ $withdraw_request_id ][ 'amount_before_charges' ];
						$current_user_wallet_balance += $amount_before_charges;

						$user_helper->ddwcwm_set_user_wallet_balance( $current_user_wallet_balance, $user_id );

						$data = [
							'type'      => 'withdraw',
							'amount'    => $amount_before_charges,
							'user_id'   => $user_id,
							'note'      => '',
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'withdraw_cancelled',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'Your withdrawal request of amount %s has been cancelled by admin. You have received the amount back in your wallet.', 'ddwc-wallet-management' ), wc_price( $amount_before_charges ) );

						if ( ! empty( $note ) ) {
							$email_message[] = sprintf( esc_html__( 'Withdrawal Cancel Note: %s', 'ddwc-wallet-management' ), $note );
						}

						$sender_email = get_userdata( $user_id )->user_email;

						$email_data = [
							'email'   => $sender_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$response = [
							'success' => true,
							'message' => sprintf( esc_html__( 'Transaction for cancelled withdrawal request of amount %s has been added successfully.', 'ddwc-wallet-management' ), wc_price( $amount_before_charges ) ),
						];

					} else if ( $perform == 'mark_as_paid' ) {
						$amount_after_charges = $withdraw_requests[ $withdraw_request_id ][ 'amount_after_charges' ];

						$data = [
							'type'      => 'withdraw',
							'amount'    => $amount_after_charges,
							'user_id'   => $user_id,
							'note'      => '',
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'withdraw_completed',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'Your withdrawal request of amount %s has been successfully completed by admin on your given payment details.', 'ddwc-wallet-management' ), wc_price( $amount_after_charges ) );

						if ( ! empty( $note ) ) {
							$email_message[] = sprintf( esc_html__( 'Withdrawal Completed Note: %s', 'ddwc-wallet-management' ), $note );
						}

						$sender_email = get_userdata( $user_id )->user_email;

						$email_data = [
							'email'   => $sender_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );

						$response = [
							'success' => true,
							'message' => sprintf( esc_html__( 'Transaction for completed withdraw request of amount %s has been added successfully.', 'ddwc-wallet-management' ), wc_price( $amount_after_charges ) ),
						];
					}

					unset( $withdraw_requests[ $withdraw_request_id ] );

					if ( ! count( $withdraw_requests ) ) {
						$user_helper->ddwcwm_delete_user_withdraw_requests( $user_id );
					} else {
						$user_helper->ddwcwm_set_user_withdraw_requests( $withdraw_requests, $user_id );
					}
				}
            } else {
				$response = [
					'success' => false,
					'message' => esc_html__( 'Security check failed!', 'ddwc-wallet-management' ),
				];
			}

			wp_send_json( $response );
			die;
		}

		/**
         * Ajax callback for importing one batch of products from a CSV.
         */
        public function ddwcwm_do_ajax_users_import() {
            check_ajax_referer( 'ddwcwm-users-import', 'security' );

            if ( ! isset( $_POST[ 'file' ] ) ) { // PHPCS: input var ok.
                wp_send_json_error( [ 'message' => esc_html__( 'Insufficient privileges to import products.', 'ddwc-wallet-management' ) ] );
            }

            $file   = wc_clean( wp_unslash( $_POST[ 'file' ] ) ); // PHPCS: input var ok.
            $params = [
                'delimiter'       => ! empty( $_POST[ 'delimiter' ] ) ? wc_clean( wp_unslash( $_POST[ 'delimiter' ] ) ) : ',', // PHPCS: input var ok.
                'start_pos'       => isset( $_POST[ 'position' ] ) ? absint( $_POST[ 'position' ] ) : 0, // PHPCS: input var ok.
                'mapping'         => isset( $_POST[ 'mapping' ] ) ? (array) wc_clean( wp_unslash( $_POST[ 'mapping' ] ) ) : [], // PHPCS: input var ok.
                'update_existing' => isset( $_POST[ 'update_existing' ] ) ? (bool) $_POST[ 'update_existing' ] : false, // PHPCS: input var ok.
                'lines'           => apply_filters( 'ddwcwm_users_import_batch_size', 30 ),
                'parse'           => true,
			];

            // Log failures.
            if ( 0 !== $params[ 'start_pos' ] ) {
                $error_log = array_filter( (array) get_user_option( 'ddwcwm_users_import_error_log' ) );
            } else {
                $error_log = [];
            }

            $importer         = DDWCWM_Users_Importer_Controller::get_importer( $file, $params );
            $results          = $importer->import();
            $percent_complete = $importer->get_percent_complete();
            $error_log        = array_merge( $error_log, $results[ 'failed' ], $results[ 'skipped' ] );

            update_user_option( get_current_user_id(), 'ddwcwm_users_import_error_log', $error_log );

            if ( 100 === $percent_complete ) {
                // Send success.
                wp_send_json_success(
                    [
                        'position'   => 'done',
                        'percentage' => 100,
                        'url'        => add_query_arg( [ '_wpnonce' => wp_create_nonce( 'ddwcwm-csv-importer' ) ], admin_url( 'admin.php?page=ddwc-wallet-management&action=ddwcwm-users-import&step=done' ) ),
                        'imported'   => count( $results[ 'imported' ] ),
                        'failed'     => count( $results[ 'failed' ] ),
                        'skipped'    => count( $results[ 'skipped' ] ),
					]
                );
            } else {
                wp_send_json_success(
                    [
                        'position'   => $importer->get_file_position(),
                        'percentage' => $percent_complete,
                        'imported'   => count( $results[ 'imported' ] ),
                        'failed'     => count( $results[ 'failed' ] ),
                        'skipped'    => count( $results[ 'skipped' ] ),
					]
                );
            }
        }

		/**
		 * AJAX callback for doing the actual users export to the CSV file.
		 * 
		 * @return void
		 */
		public function ddwcwm_do_ajax_users_export() {
			check_ajax_referer( 'ddwcwm-users-export', 'security' );

			$step     = isset( $_POST[ 'step' ] ) ? absint( $_POST[ 'step' ] ) : 1; // WPCS: input var ok, sanitization ok.
			$exporter = new DDWCWM_Users_CSV_Exporter();

			if ( ! empty( $_POST[ 'columns' ] ) ) { // WPCS: input var ok.
				$exporter->set_column_names( wp_unslash( $_POST[ 'columns' ] ) ); // WPCS: input var ok, sanitization ok.
			}

			if ( ! empty( $_POST[ 'selected_columns' ] ) ) { // WPCS: input var ok.
				$exporter->set_columns_to_export( wp_unslash( $_POST[ 'selected_columns' ] ) ); // WPCS: input var ok, sanitization ok.
			}

			if ( ! empty( $_POST[ 'filename' ] ) ) { // WPCS: input var ok.
				$exporter->set_filename( wp_unslash( $_POST[ 'filename' ] ) ); // WPCS: input var ok, sanitization ok.
			}

			$exporter->set_page( $step );
			$exporter->generate_file();

			$query_args = [
				'nonce'    => wp_create_nonce( 'users-csv' ),
				'handle'   => 'download_users_csv',
				'filename' => $exporter->get_filename(),
			];

			if ( 100 === $exporter->get_percent_complete() ) {
				wp_send_json_success( [
					'step'       => 'done',
					'percentage' => 100,
					'url'        => add_query_arg( $query_args, admin_url( 'admin.php?page=ddwc-wallet-management&action=ddwcwm-users-export' ) ),
				] );
			} else {
				wp_send_json_success( [
					'step'       => ++$step,
					'percentage' => $exporter->get_percent_complete(),
					'columns'    => $exporter->get_column_names(),
				] );
			}
		}
	}
}
