<?php
/**
 * Class DDWCWM_Users_Importer_Controller file.
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes\Admin\Importer;

use DDWCWalletManagement\Includes\Admin\Importer\DDWCWM_Users_CSV_Importer;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'WP_Importer' ) ) {
	return;
}

/**
 * Outlet importer controller - handles file upload and forms in admin.
 *
 * @package     WooCommerce/Admin/Importers
 * @version     3.1.0
 */
class DDWCWM_Users_Importer_Controller {

	/**
	 * The path to the current file.
	 *
	 * @var string
	 */
	protected $file = '';

	/**
	 * The current import step.
	 *
	 * @var string
	 */
	protected $step = '';

	/**
	 * Progress steps.
	 *
	 * @var array
	 */
	protected $steps = [];

	/**
	 * Errors.
	 *
	 * @var array
	 */
	protected $errors = [];

	/**
	 * The current delimiter for the file being read.
	 *
	 * @var string
	 */
	protected $delimiter = ',';

	/**
	 * Whether to use previous mapping selections.
	 *
	 * @var bool
	 */
	protected $map_preferences = false;

	/**
	 * Whether to skip existing users.
	 *
	 * @var bool
	 */
	protected $update_existing = false;

	/**
	 * Get importer instance.
	 *
	 * @param  string $file File to import.
	 * @param  array  $args Importer arguments.
	 * @return DDWCWM_Users_CSV_Importer
	 */
	public static function get_importer( $file, $args = [] ) {
        $args = apply_filters( 'ddwcwm_users_csv_importer_args', $args, 'DDWCWM_Users_CSV_Importer' );
		return new DDWCWM_Users_CSV_Importer( $file, $args );
	}

	/**
	 * Check whether a file is a valid CSV file.
	 *
	 * @todo Replace this method with wc_is_file_valid_csv() function.
	 * @param string $file File path.
	 * @param bool   $check_path Whether to also check the file is located in a valid location (Default: true).
	 * @return bool
	 */
	public static function is_file_valid_csv( $file, $check_path = true ) {
		if ( $check_path && apply_filters( 'ddwcwm_users_csv_importer_check_import_file_path', true ) && false !== stripos( $file, '://' ) ) {
			return false;
		}

		$valid_filetypes = self::get_valid_csv_filetypes();
		$filetype        = wp_check_filetype( $file, $valid_filetypes );
		if ( in_array( $filetype[ 'type' ], $valid_filetypes, true ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Get all the valid filetypes for a CSV file.
	 *
	 * @return array
	 */
	protected static function get_valid_csv_filetypes() {
		return apply_filters(
			'ddwcwm_csv_users_import_valid_filetypes',
			[
				'csv' => 'text/csv',
				'txt' => 'text/plain',
            ]
		);
	}

	/**
	 * Constructor.
	 */
	public function __construct() {
		$default_steps = [
			'upload'  => [
				'name'    => esc_html__( 'Upload CSV file', 'ddwc-wallet-management' ),
				'view'    => [ $this, 'upload_form' ],
				'handler' => [ $this, 'upload_form_handler' ],
			],
			'mapping' => [
				'name'    => esc_html__( 'Column mapping', 'ddwc-wallet-management' ),
				'view'    => [ $this, 'mapping_form' ],
				'handler' => '',
			],
			'import'  => [
				'name'    => esc_html__( 'Import', 'ddwc-wallet-management' ),
				'view'    => [ $this, 'import' ],
				'handler' => '',
			],
			'done'    => [
				'name'    => esc_html__( 'Done!', 'ddwc-wallet-management' ),
				'view'    => [ $this, 'done' ],
				'handler' => '',
			],
		];

		$this->steps = apply_filters( 'ddwcwm_users_csv_importer_steps', $default_steps );

		// phpcs:disable WordPress.Security.NonceVerification.NoNonceVerification
		$this->step            = isset( $_REQUEST[ 'step' ] ) ? sanitize_key( $_REQUEST[ 'step' ] ) : current( array_keys( $this->steps ) );
		$this->file            = isset( $_REQUEST[ 'file' ] ) ? wc_clean( wp_unslash( $_REQUEST[ 'file' ] ) ) : '';
		$this->update_existing = isset( $_REQUEST[ 'update_existing' ] ) ? (bool) $_REQUEST[ 'update_existing' ] : false;
		$this->delimiter       = ! empty( $_REQUEST[ 'delimiter' ] ) ? wc_clean( wp_unslash( $_REQUEST[ 'delimiter' ] ) ) : ',';
		$this->map_preferences = isset( $_REQUEST[ 'map_preferences' ] ) ? (bool) $_REQUEST[ 'map_preferences' ] : false;
		// phpcs:enable

		if ( $this->map_preferences ) {
			add_filter( 'ddwcwm_csv_users_import_mapped_columns', [ $this, 'auto_map_user_preferences' ], 9999 );
		}
	}

	/**
	 * Get the URL for the next step's screen.
	 *
	 * @param string $step  slug (default: current step).
	 * @return string       URL for next step if a next step exists.
	 *                      Admin URL if it's the last step.
	 *                      Empty string on failure.
	 */
	public function get_next_step_link( $step = '' ) {
		if ( ! $step ) {
			$step = $this->step;
		}

		$keys = array_keys( $this->steps );

		if ( end( $keys ) === $step ) {
			return admin_url();
		}

		$step_index = array_search( $step, $keys, true );

		if ( false === $step_index ) {
			return '';
		}

		$params = [
			'step'            => $keys[ $step_index + 1 ],
			'file'            => str_replace( DIRECTORY_SEPARATOR, '/', $this->file ),
			'delimiter'       => $this->delimiter,
			'update_existing' => $this->update_existing,
			'map_preferences' => $this->map_preferences,
			'_wpnonce'        => wp_create_nonce( 'ddwcwm-csv-importer' ), // wp_nonce_url() escapes & to &amp; breaking redirects.
		];

		return add_query_arg( $params );
	}

	/**
	 * Output header view.
	 */
	protected function output_header() {
		include dirname( __FILE__ ) . '/views/html-csv-import-header.php';
	}

	/**
	 * Output steps view.
	 */
	protected function output_steps() {
		include dirname( __FILE__ ) . '/views/html-csv-import-steps.php';
	}

	/**
	 * Output footer view.
	 */
	protected function output_footer() {
		include dirname( __FILE__ ) . '/views/html-csv-import-footer.php';
	}

	/**
	 * Add error message.
	 *
	 * @param string $message Error message.
	 * @param array  $actions List of actions with 'url' and 'label'.
	 */
	protected function add_error( $message, $actions = [] ) {
		$this->errors[] = [
			'message' => $message,
			'actions' => $actions,
        ];
	}

	/**
	 * Add error message.
	 */
	protected function output_errors() {
		if ( ! $this->errors ) {
			return;
		}

		foreach ( $this->errors as $error ) {
			echo '<div class="error inline">';
			echo '<p>' . esc_html( $error[ 'message' ] ) . '</p>';

			if ( ! empty( $error[ 'actions' ] ) ) {
				echo '<p>';
				foreach ( $error[ 'actions' ] as $action ) {
					echo '<a class="button button-primary" href="' . esc_url( $action[ 'url' ] ) . '">' . esc_html( $action[ 'label' ] ) . '</a> ';
				}
				echo '</p>';
			}
			echo '</div>';
		}
	}

	/**
	 * Dispatch current step and show correct view.
	 */
	public function dispatch() {
		// phpcs:ignore WordPress.Security.NonceVerification.NoNonceVerification
		if ( ! empty( $_POST[ 'save_step' ] ) && ! empty( $this->steps[ $this->step ][ 'handler' ] ) ) {
			call_user_func( $this->steps[ $this->step ][ 'handler' ], $this );
		}
		$this->output_header();
		$this->output_steps();
		$this->output_errors();
		call_user_func( $this->steps[ $this->step ][ 'view' ], $this );
		$this->output_footer();
	}

	/**
	 * Output information about the uploading process.
	 */
	protected function upload_form() {
		$bytes      = apply_filters( 'import_upload_size_limit', wp_max_upload_size() );
		$size       = size_format( $bytes );
		$upload_dir = wp_upload_dir();

		include dirname( __FILE__ ) . '/views/html-users-csv-import-form.php';
	}

	/**
	 * Handle the upload form and store options.
	 */
	public function upload_form_handler() {
		check_admin_referer( 'ddwcwm-csv-importer' );

		$file = $this->handle_upload();

		if ( is_wp_error( $file ) ) {
			$this->add_error( $file->get_error_message() );
			return;
		} else {
			$this->file = $file;
		}

		wp_redirect( esc_url_raw( $this->get_next_step_link() ) );
		exit;
	}

	/**
	 * Handles the CSV upload and initial parsing of the file to prepare for
	 * displaying author import options.
	 *
	 * @return string|WP_Error
	 */
	public function handle_upload() {
		// phpcs:disable WordPress.Security.NonceVerification.NoNonceVerification -- Nonce already verified in DDWCWM_Users_Importer_Controller::upload_form_handler()
		$file_url = isset( $_POST[ 'file_url' ] ) ? wc_clean( wp_unslash( $_POST[ 'file_url' ] ) ) : '';

		if ( empty( $file_url ) ) {
			if ( ! isset( $_FILES[ 'import' ] ) ) {
				return new \WP_Error( 'ddwcwm_users_csv_importer_upload_file_empty', esc_html__( 'File is empty. Please upload something more substantial. This error could also be caused by uploads being disabled in your php.ini or by post_max_size being defined as smaller than upload_max_filesize in php.ini.', 'ddwc-wallet-management' ) );
			}

			if ( ! self::is_file_valid_csv( wc_clean( wp_unslash( $_FILES[ 'import' ][ 'name' ] ) ), false ) ) {
				return new \WP_Error( 'ddwcwm_users_csv_importer_upload_file_invalid', esc_html__( 'Invalid file type. The importer supports CSV and TXT file formats.', 'ddwc-wallet-management' ) );
			}

			$overrides = [
				'test_form' => false,
				'mimes'     => self::get_valid_csv_filetypes(),
			];
			$import    = $_FILES[ 'import' ]; // WPCS: sanitization ok, input var ok.
			$upload    = wp_handle_upload( $import, $overrides );

			if ( isset( $upload[ 'error' ] ) ) {
				return new \WP_Error( 'ddwcwm_users_csv_importer_upload_error', $upload[ 'error' ] );
			}

			// Construct the object array.
			$object = [
				'post_title'     => basename( $upload[ 'file' ] ),
				'post_content'   => $upload[ 'url' ],
				'post_mime_type' => $upload[ 'type' ],
				'guid'           => $upload[ 'url' ],
				'context'        => 'import',
				'post_status'    => 'private',
			];

			// Save the data.
			$id = wp_insert_attachment( $object, $upload[ 'file' ] );

			/*
			 * Schedule a cleanup for one day from now in case of failed
			 * import or missing wp_import_cleanup() call.
			 */
			wp_schedule_single_event( time() + DAY_IN_SECONDS, 'importer_scheduled_cleanup', [ $id ] );

			return $upload[ 'file' ];
		} elseif ( file_exists( ABSPATH . $file_url ) ) {
			if ( ! self::is_file_valid_csv( ABSPATH . $file_url ) ) {
				return new \WP_Error( 'ddwcwm_users_csv_importer_upload_file_invalid', esc_html__( 'Invalid file type. The importer supports CSV and TXT file formats.', 'ddwc-wallet-management' ) );
			}

			return ABSPATH . $file_url;
		}
		// phpcs:enable

		return new \WP_Error( 'ddwcwm_users_csv_importer_upload_invalid_file', esc_html__( 'Please upload or provide the link to a valid CSV file.', 'ddwc-wallet-management' ) );
	}

	/**
	 * Mapping step.
	 */
	protected function mapping_form() {
		check_admin_referer( 'ddwcwm-csv-importer' );
		$args = [
			'lines'     => 1,
			'delimiter' => $this->delimiter,
		];

		$importer     = self::get_importer( $this->file, $args );
		$headers      = $importer->get_raw_keys();
		$mapped_items = $this->auto_map_columns( $headers );
		$sample       = current( $importer->get_raw_data() );

		if ( empty( $sample ) ) {
			$this->add_error(
				esc_html__( 'The file is empty or using a different encoding than UTF-8, please try again with a new file.', 'ddwc-wallet-management' ),
				[
					[
						'url'   => esc_url( admin_url( "admin.php?page=${$_GET['page']}&action=ddwcwm-users-import" ) ),
						'label' => esc_html__( 'Upload a new file', 'ddwc-wallet-management' ),
                    ],
				]
			);

			// Force output the errors in the same page.
			$this->output_errors();
			return;
		}

		include_once dirname( __FILE__ ) . '/views/html-csv-import-mapping.php';
	}

	/**
	 * Import the file if it exists and is valid.
	 */
	public function import() {
		// Displaying this page triggers Ajax action to run the import with a valid nonce,
		// therefore this page needs to be nonce protected as well.
		check_admin_referer( 'ddwcwm-csv-importer' );

		if ( ! self::is_file_valid_csv( $this->file ) ) {
			$this->add_error( __( 'Invalid file type. The importer supports CSV and TXT file formats.', 'ddwc-wallet-management' ) );
			$this->output_errors();
			return;
		}

		if ( ! is_file( $this->file ) ) {
			$this->add_error( __( 'The file does not exist, please try again.', 'ddwc-wallet-management' ) );
			$this->output_errors();
			return;
		}

		if ( ! empty( $_POST[ 'map_from' ] ) && ! empty( $_POST[ 'map_to' ] ) ) {
			$mapping_from = wc_clean( wp_unslash( $_POST[ 'map_from' ] ) );
			$mapping_to   = wc_clean( wp_unslash( $_POST[ 'map_to' ] ) );

			// Save mapping preferences for future imports.
			update_user_option( get_current_user_id(), 'ddwcwm_users_import_mapping', $mapping_to );
		} else {
			wp_redirect( esc_url_raw( $this->get_next_step_link( 'upload' ) ) );
			exit;
		}

		wp_enqueue_script( 'ddwcwm-users-import' );

		wp_localize_script(
			'ddwcwm-users-import',
			'ddwcwm_users_import_params',
			[
				'import_nonce'    => wp_create_nonce( 'ddwcwm-users-import' ),
				'mapping'         => [
					'from' => $mapping_from,
					'to'   => $mapping_to,
                ],
				'file'            => $this->file,
				'update_existing' => $this->update_existing,
				'delimiter'       => $this->delimiter,
            ]
		);

		include_once dirname( __FILE__ ) . '/views/html-csv-import-progress.php';
	}

	/**
	 * Done step.
	 */
	protected function done() {
		check_admin_referer( 'ddwcwm-csv-importer' );
		$imported = isset( $_GET[ 'users-imported' ] ) ? absint( $_GET[ 'users-imported' ] ) : 0;
		$updated  = isset( $_GET[ 'users-updated' ] ) ? absint( $_GET[ 'users-updated' ] ) : 0;
		$failed   = isset( $_GET[ 'users-failed' ] ) ? absint( $_GET[ 'users-failed' ] ) : 0;
		$skipped  = isset( $_GET[ 'users-skipped' ] ) ? absint( $_GET[ 'users-skipped' ] ) : 0;
		$errors   = array_filter( (array) get_user_option( 'ddwcwm_users_import_error_log' ) );

		include_once dirname( __FILE__ ) . '/views/html-csv-import-done.php';
	}

	/**
	 * Columns to normalize.
	 *
	 * @param  array $columns List of columns names and keys.
	 * @return array
	 */
	protected function normalize_columns_names( $columns ) {
		$normalized = [];

		foreach ( $columns as $key => $value ) {
			$normalized[ strtolower( $key ) ] = $value;
		}

		return $normalized;
	}

	/**
	 * Auto map column names.
	 *
	 * @param  array $raw_headers Raw header columns.
	 * @param  bool  $num_indexes If should use numbers or raw header columns as indexes.
	 * @return array
	 */
	protected function auto_map_columns( $raw_headers, $num_indexes = true ) {

		/*
		 * @hooked ddwcwm_importer_generic_mappings - 10
		 * @hooked ddwcwm_importer_wordpress_mappings - 10
		 * @hooked ddwcwm_importer_default_english_mappings - 100
		 */
		$default_columns = $this->normalize_columns_names(
			apply_filters(
				'ddwcwm_csv_users_import_mapping_default_columns',
				[
                    esc_html__( 'Username', 'ddwc-wallet-management' ) => 'username',
                    esc_html__( 'Amount', 'ddwc-wallet-management' )   => 'amount',
                    esc_html__( 'Action', 'ddwc-wallet-management' )   => 'action',
                    esc_html__( 'Note', 'ddwc-wallet-management' )     => 'note',
                ],
				$raw_headers
			)
		);

		$headers = [];
		foreach ( $raw_headers as $key => $field ) {
			$field             = strtolower( $field );
			$index             = $num_indexes ? $key : $field;
			$headers[ $index ] = $field;

			if ( isset( $default_columns[ $field ] ) ) {
				$headers[ $index ] = $default_columns[ $field ];
			}
		}

		return apply_filters( 'ddwcwm_csv_users_import_mapped_columns', $headers, $raw_headers );
	}

	/**
	 * Map columns using the user's lastest import mappings.
	 *
	 * @param  array $headers Header columns.
	 * @return array
	 */
	public function auto_map_user_preferences( $headers ) {
		$mapping_preferences = get_user_option( 'ddwcwm_users_import_mapping' );

		if ( ! empty( $mapping_preferences ) && is_array( $mapping_preferences ) ) {
			return $mapping_preferences;
		}

		return $headers;
	}

	/**
	 * Sanitize special column name regex.
	 *
	 * @param  string $value Raw special column name.
	 * @return string
	 */
	protected function sanitize_special_column_name_regex( $value ) {
		return '/' . str_replace( [ '%d', '%s' ], '(.*)', trim( quotemeta( $value ) ) ) . '/';
	}

	/**
	 * Get special columns.
	 *
	 * @param  array $columns Raw special columns.
	 * @return array
	 */
	protected function get_special_columns( $columns ) {
		$formatted = [];

		foreach ( $columns as $key => $value ) {
			$regex = $this->sanitize_special_column_name_regex( $key );

			$formatted[ $regex ] = $value;
		}

		return $formatted;
	}

	/**
	 * Get mapping options.
	 *
	 * @param  string $item Item name.
	 * @return array
	 */
	protected function get_mapping_options( $item = '' ) {
		// Get index for special column names.
		$index = $item;

		if ( preg_match( '/\d+/', $item, $matches ) ) {
			$index = $matches[0];
		}

		// Available options.
		$options = [
			'username' => esc_html__( 'Username', 'ddwc-wallet-management' ),
			'amount'   => esc_html__( 'Amount', 'ddwc-wallet-management' ),
			'action'   => esc_html__( 'Action', 'ddwc-wallet-management' ),
			'note'     => esc_html__( 'Note', 'ddwc-wallet-management' ),
        ];

        return apply_filters( 'ddwcwm_csv_users_import_mapping_options', $options, $item );
	}
}
