<?php
/**
 * Common functions class
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes\Common;

use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;
use DDWCWalletManagement\Includes\DDWCWM_Email_Notification_Handler;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Common_Functions' ) ) {
	/**
	 * Common functions
	 */
	class DDWCWM_Common_Functions {

		/**
         * Register new endpoint to use inside My Account page.
		 *
		 * @return void
		 */
        public function ddwcwm_add_endpoints() {
			global $ddwcwm_wallet;
			add_rewrite_endpoint( $ddwcwm_wallet[ 'my_account_endpoint' ], EP_ROOT | EP_PAGES );
		}

		/**
		 * Add Email Notification function
		 *
		 * @param array $email_classes
		 * @return array
		 */
		public function ddwcwm_add_new_email_notification( $email_classes ) {
			$email_classes[ 'WC_Email_Wallet_Management_Notification' ] = new DDWCWM_Email_Notification_Handler();
            return $email_classes;
		}

		/**
		 * Add Email Notification Action function
		 *
		 * @param array $actions
		 * @return array
		 */
		public function ddwcwm_add_notification_actions( $actions ) {
			$actions[] = 'ddwcwm_mail';
            return $actions;
        }

        /**
         * Add the gateway to woocommerce
         *
         * @param array $methods All payment methods.
         */
		public function ddwcwm_add_payment_gateway( $methods ) {
			$methods[] = 'DDWCWalletManagement\Includes\DDWCWM_Wallet_Gateway';
            return $methods;
		}

		/**
		 * Add whatsapp automation function
		 *
		 * @param string $whatsapp_message
		 * @param object $response
		 * @return string
		 */
		public function ddwcwm_add_whatsapp_automation( $whatsapp_message, $response ) {
			$twilio_request    = $response->get_params();
			$user_message_body = $twilio_request[ 'Body' ];

			if ( strpos( strtolower( $user_message_body ), 'wallet' ) !== false ) {
				$whatsapp_from    = str_replace( 'whatsapp:', '', $twilio_request[ 'From' ] );

				$user_id = $this->ddwcwm_get_user_id_by_billing_phone( $whatsapp_from );

				if ( ! empty( $user_id ) ) {
					$user_helper      = new DDWCWM_Users_Helper();
					$wallet_balance   = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );
					$whatsapp_message = sprintf( esc_html__( 'Your current wallet balance is %s', 'ddwc-wallet-management' ), $this->ddwcwm_whatsapp_price( $wallet_balance ) );
				} else {
                    $permalink = get_permalink( get_option( 'woocommerce_myaccount_page_id' ) ) . '/' . get_option( 'woocommerce_myaccount_edit_account_endpoint', 'edit-account' ) . '/billing';

					$whatsapp_message = sprintf( esc_html__( 'Sorry, we are not able to find your number saved on your account, kindly save your phone number on the billing phone field from here - %s', 'ddwc-wallet-management' ), esc_url( $permalink ) );
				}
			} else {
				$whatsapp_message .= "\n\n" . html_entity_decode( esc_html__( 'You can also check your wallet balance from here just by typing like "What is my wallet balance?"', 'ddwc-wallet-management' ) );
			}

			return $whatsapp_message;
		}

		/**
         * Get price with currency and without html function
         *
         * @param float|string $amount
         * @return string
         */
        public function ddwcwm_whatsapp_price( $amount ) {
            return html_entity_decode( wp_strip_all_tags( wc_price( $amount ) ) );
        }

		/**
         * Get user id by billing phone function
         *
         * @param string $billing_phone
         * @return integer|null
         */
        public function ddwcwm_get_user_id_by_billing_phone( $billing_phone ) {
			global $wpdb;
			return $wpdb->get_var( $wpdb->prepare( "SELECT user_id FROM $wpdb->usermeta WHERE meta_key=%s AND meta_value=%s", 'billing_phone', $billing_phone ) );
        }

		/**
		 * Handle removing wallet for whatsapp purchase function
		 *
		 * @param boolean $initial
		 * @param object $gateway
		 * @param object $response
		 * @return boolean
		 */
		public function ddwcwm_handle_removing_wallet_for_whatsapp_purchase( $initial, $gateway, $cart_total, $response ) {
			if ( 'ddwcwm_wallet' === $gateway->id ) {
				global $ddwcwm_wallet;
				$twilio_request    = $response->get_params();
				$user_message_body = $twilio_request[ 'Body' ];
				$whatsapp_from     = str_replace( 'whatsapp:', '', $twilio_request[ 'From' ] );
				$user_id           = $this->ddwcwm_get_user_id_by_billing_phone( $whatsapp_from );

				if ( ! empty( $user_id ) ) {
					$user_helper    = new DDWCWM_Users_Helper();
					$wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );

					if ( $wallet_balance <= 0 || $wallet_balance < $cart_total || ( ! empty( $ddwcwm_wallet[ 'max_debit_limit' ] ) && $ddwcwm_wallet[ 'max_debit_limit' ] < $cart_total ) ) {
						return false;
					}

					return true;
				} else {
                    return false;
				}
			}

			return $initial;
		}

	}

}
