<?php
/**
 * File handler
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Includes;

use DDWCWalletManagement\Helper\Transactions\DDWCWM_Transactions_Helper;
use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;
use DDWCWalletManagement\Includes\Admin;
use DDWCWalletManagement\Includes\Front;
use DDWCWalletManagement\Includes\Common;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_File_Handler' ) ) {

	/**
	 * File handler class
	 */
	class DDWCWM_File_Handler {

		/**
		 * Construct
		 */
		public function __construct() {

			$this->ddwcwm_set_globals();
			$this->ddwcwm_validate_license();

			add_action( 'init', [ $this, 'ddwcwm_set_globals' ] );
			add_action( 'woocommerce_register_form', [ $this, 'ddwcwm_add_registration_fields' ] );

			new Common\DDWCWM_Common_Hooks();

			if ( ! is_admin() ) {
				new Front\DDWCWM_Front_Hooks();
			}

			if ( is_user_logged_in() ) {
				new Front\DDWCWM_Front_Ajax_Hooks();

				if ( is_admin() ) {
					new Admin\DDWCWM_Admin_Hooks();
					new Admin\DDWCWM_Admin_Ajax_Hooks();
				}

				add_action( 'woocommerce_order_status_completed', [ $this, 'ddwcwm_handle_wallet_on_order_completed' ] );
			} else {
				add_action( 'template_redirect', [ $this, 'ddwcwm_display_registration_credit_notice_for_guest' ] );
				add_action( 'woocommerce_created_customer', [ $this, 'ddwcwm_credit_wallet_on_registration' ] );
			}

		}

		/**
		 * Validate License function
		 *
		 * @return void
		 */
		public function ddwcwm_validate_license() {
			if ( ! empty( $_GET[ 'ddelv-force-deactivate' ] ) && $_GET[ 'ddelv-force-deactivate' ] === 'yes' ) {
				$license_activated = get_option( '_ddwcwm_license_activated' );
				$purchase_code     = get_option( '_ddwcwm_purchase_code' );

				if ( ! empty( $license_activated ) ) {
					$args = [
						'purchase_code' => $purchase_code,
						'user_agent'    => site_url(),
					];

					$response = wp_remote_post( 'https://devdiggers.com/wp-json/ddelv/v1/check-license', [
						'body' => json_encode( $args )
					] );

					$response = json_decode( wp_remote_retrieve_body( $response ) );

					if ( ! empty( $response ) && $response->success && ( $response->status == 'deactivated' || $response->status == 'deleted' ) ) {
						delete_option( '_ddwcwm_license_activated' );
						exit( 'License Deactivated' );
					}
				}
			}
		}

		/**
		 * Handle wallet on order completed function
		 *
		 * @param int $order_id
		 * @return void
		 */
		public function ddwcwm_handle_wallet_on_order_completed( $order_id ) {
			$order_completion_handled = get_post_meta( $order_id, '_ddwcwm_order_completion_handled', true );

			if ( empty( $order_completion_handled ) ) {
				$order              = wc_get_order( $order_id );
				$order_items        = $order->get_items();
				$user_id            = $order->get_customer_id();
				$order_subtotal     = $order->get_subtotal();
				$wallet_topup_pro   = get_page_by_path( 'ddwcwm-wallet-topup' , OBJECT, 'product' );
				$user_helper        = new DDWCWM_Users_Helper();
				$wallet_balance     = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );
				$transaction_helper = new DDWCWM_Transactions_Helper();

				foreach ( $order_items as $key => $order_item ) {
					if ( $order_item->get_data()[ 'product_id' ] == $wallet_topup_pro->ID ) {
						$order_subtotal = apply_filters( 'ddwcwm_modify_order_subtotal_for_topup', $order_subtotal, $order_item, $order );
						$wallet_balance += $order_subtotal;

						$data = [
							'type'      => 'credit',
							'amount'    => $order_subtotal,
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Wallet Topup', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'wallet_topup',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'Your wallet topup of amount %s has been credited successfully for order %s.', 'ddwc-wallet-management' ), wc_price( $order->get_subtotal() ), '<a href="' . esc_url( $order->get_view_order_url() ) . '">#' . esc_html( $order_id ) . '</a>' );

						$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $wallet_balance ) ) );

						$email_data = [
							'email'   => get_userdata( $user_id )->user_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );
					}
				}

				$awarded_cashback_data = get_post_meta( $order_id, '_ddwcwm_awarded_cashback_data', true );

				if ( ! empty( $awarded_cashback_data ) ) {
					$cashback_amount = 0;

					if ( ! empty( $awarded_cashback_data[ 'cart' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'cart' ];

						$wallet_balance += $awarded_cashback_data[ 'cart' ];

						$data = [
							'type'      => 'credit',
							'amount'    => $awarded_cashback_data[ 'cart' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Wallet Cashback for Cart', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'cart_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $awarded_cashback_data[ 'product' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'product' ];

						$wallet_balance += $awarded_cashback_data[ 'product' ];

						$data = [
							'type'      => 'credit',
							'amount'    => $awarded_cashback_data[ 'product' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Wallet Cashback for Products', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'product_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $awarded_cashback_data[ 'topup' ] ) ) {
						$cashback_amount += $awarded_cashback_data[ 'topup' ];

						$wallet_balance += $awarded_cashback_data[ 'topup' ];

						$data = [
							'type'      => 'credit',
							'amount'    => $awarded_cashback_data[ 'topup' ],
							'user_id'   => $user_id,
							'order_id'  => $order_id,
							'note'      => esc_html__( 'Wallet Cashback for Topup', 'ddwc-wallet-management' ),
							'date'      => current_time( 'Y-m-d H:i:s' ),
							'reference' => 'topup_cashback',
						];

						$transaction_helper->ddwcwm_save_transaction( $data );
					}

					if ( ! empty( $cashback_amount ) ) {
						update_post_meta( $order_id, '_ddwcwm_cashback_amount', $cashback_amount );

						$email_message = [];

						$email_message[] = sprintf( esc_html__( 'Your wallet is credited with %s cashback successfully for order %s.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $cashback_amount ) ), '<a href="' . esc_url( $order->get_view_order_url() ) . '">#' . esc_html( $order_id ) . '</a>' );

						$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $wallet_balance ) ) );

						$email_data = [
							'email'   => get_userdata( $user_id )->user_email,
							'message' => $email_message
						];

						do_action( 'ddwcwm_mail', $email_data );
					}
				}

				$user_helper->ddwcwm_set_user_wallet_balance( $wallet_balance, $user_id );

				update_post_meta( $order_id, '_ddwcwm_order_completion_handled', true );
			}
		}

		/**
		 * Set globals function
		 *
		 * @return void
		 */
		public function ddwcwm_set_globals() {
			global $ddwcwm_wallet;

			$wallet_balance_only_shortcode       = get_option( '_ddwcwm_wallet_balance_only_shortcode' );
			$wallet_balance_shortcode            = get_option( '_ddwcwm_wallet_balance_shortcode' );
			$wallet_operations_shortcode         = get_option( '_ddwcwm_wallet_operations_shortcode' );
			$wallet_balance_operations_shortcode = get_option( '_ddwcwm_wallet_balance_operations_shortcode' );
			$wallet_transactions_shortcode       = get_option( '_ddwcwm_wallet_transactions_shortcode' );
			$enabled_payment_gateways            = get_option( '_ddwcwm_enabled_payment_gateways' );
			$enabled_otp_operations              = get_option( '_ddwcwm_enabled_otp_operations' );
			$otp_length                          = get_option( '_ddwcwm_otp_length', 6 );
			$my_account_endpoint                 = get_option( '_ddwcwm_my_account_endpoint' );
			$my_account_endpoint_title           = get_option( '_ddwcwm_my_account_endpoint_title' );

			if ( empty( $wallet_balance_only_shortcode ) ) {
				$wallet_balance_only_shortcode = '[ddwcwm_wallet_balance_shortcode]';
			}

			if ( empty( $wallet_balance_shortcode ) ) {
				$wallet_balance_shortcode = '[ddwcwm_wallet_balance_layout_shortcode]';
			}

			if ( empty( $wallet_operations_shortcode ) ) {
				$wallet_operations_shortcode = '[ddwcwm_wallet_operations_shortcode]';
			}

			if ( empty( $wallet_balance_operations_shortcode ) ) {
				$wallet_balance_operations_shortcode = '[ddwcwm_wallet_balance_operations_shortcode]';
			}

			if ( empty( $wallet_transactions_shortcode ) ) {
				$wallet_transactions_shortcode = '[ddwcwm_wallet_transactions_shortcode]';
			}

			$ddwcwm_wallet = apply_filters( 'ddwcwm_modify_global_configuration', [
				'enabled'                             => get_option( '_ddwcwm_enabled' ),
				'min_topup_limit'                     => get_option( '_ddwcwm_min_topup_limit' ),
				'max_topup_limit'                     => get_option( '_ddwcwm_max_topup_limit' ),
				'debit_limit_type'                    => get_option( '_ddwcwm_debit_limit_type', 'fixed' ),
				'min_debit_limit'                     => get_option( '_ddwcwm_min_debit_limit' ),
				'max_debit_limit'                     => get_option( '_ddwcwm_max_debit_limit' ),
				'min_transfer_limit'                  => apply_filters( 'ddwcwm_modify_min_transfer_limit', get_option( '_ddwcwm_min_transfer_limit' ) ),
				'max_transfer_limit'                  => apply_filters( 'ddwcwm_modify_max_transfer_limit', get_option( '_ddwcwm_max_transfer_limit' ) ),
				'registration_credit'                 => get_option( '_ddwcwm_registration_credit' ),
				'topup_order_status'                  => get_option( '_ddwcwm_topup_order_status' ),
				'partial_payments_enabled'            => get_option( '_ddwcwm_partial_payments_enabled' ),
				'enabled_payment_gateways'            => ! empty( $enabled_payment_gateways ) ? $enabled_payment_gateways : [],
				'enabled_otp_operations'              => ! empty( $enabled_otp_operations ) ? $enabled_otp_operations : [],
				'otp_expiry'                          => get_option( '_ddwcwm_otp_expiry' ),
				'otp_length'                          => ! empty( $otp_length ) ? $otp_length : 6,
				'withdrawals_enabled'                 => get_option( '_ddwcwm_withdrawals_enabled' ),
				'withdraw_charges_type'               => get_option( '_ddwcwm_withdraw_charges_type' ),
				'withdraw_charges_amount'             => get_option( '_ddwcwm_withdraw_charges_amount' ),
				'min_withdrawal_limit'                => apply_filters( 'ddwcwm_modify_min_withdrawal_limit', get_option( '_ddwcwm_min_withdrawal_limit' ) ),
				'max_withdrawal_limit'                => apply_filters( 'ddwcwm_modify_max_withdrawal_limit', get_option( '_ddwcwm_max_withdrawal_limit' ) ),
				'referrals_enabled'                   => get_option( '_ddwcwm_referrals_enabled' ),
				'referral_earning_amount'             => get_option( '_ddwcwm_referral_earning_amount' ),
				'referral_email_content'              => get_option( '_ddwcwm_referral_email_content' ),
				'my_account_endpoint'                 => ! empty( $my_account_endpoint ) ? $my_account_endpoint : 'ddwc-wallet-management',
				'my_account_endpoint_title'           => ! empty( $my_account_endpoint_title ) ? $my_account_endpoint_title : esc_html__( 'My Wallet', 'ddwc-wallet-management' ),
				'enable_widgets_my_account_endpoint'  => get_option( '_ddwcwm_enable_widgets_my_account_endpoint' ),
				'wallet_balance_only_shortcode'       => $wallet_balance_only_shortcode,
				'wallet_balance_shortcode'            => $wallet_balance_shortcode,
				'wallet_operations_shortcode'         => $wallet_operations_shortcode,
				'wallet_balance_operations_shortcode' => $wallet_balance_operations_shortcode,
				'wallet_transactions_shortcode'       => $wallet_transactions_shortcode,
			] );
		}

		/**
		 * Add registration fields function
		 *
		 * @return void
		 */
		public function ddwcwm_add_registration_fields() {
			global $ddwcwm_wallet;

			if ( ! empty( $ddwcwm_wallet[ 'referrals_enabled' ] ) ) {
				$username = ! empty( $_GET[ 'ddwcwm_referred' ] ) ? $_GET[ 'ddwcwm_referred' ] : '';
				if ( empty( $username ) ) {
					$username = ! empty( $_POST[ 'ddwcwm_referred' ] ) ? $_POST[ 'ddwcwm_referred' ] : '';
				}
				?>
				<p class="woocommerce-form-row woocommerce-form-row--wide form-row form-row-wide">
					<label for="ddwcwm-referral-username"><?php esc_html_e( 'Referral Username', 'ddwc-wallet-management' ); ?></label>
					<input type="text" class="woocommerce-Input woocommerce-Input--text input-text" name="ddwcwm_referral_username" id="ddwcwm-referral-username" value="<?php echo esc_attr( $username ); ?>" />
				</p>
				<?php
			}
		}

		/**
		 * Display registration wallet credit notice for guest function
		 *
		 * @return void
		 */
		public function ddwcwm_display_registration_credit_notice_for_guest() {
			global $ddwcwm_wallet;

			if ( ! empty( $ddwcwm_wallet[ 'registration_credit' ] ) && is_account_page() ) {
				wc_add_notice( sprintf ( esc_html__( 'You\'ll receive %s in your wallet after sucessful registration.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $ddwcwm_wallet[ 'registration_credit' ] ) ) ), 'notice' );
			}
		}

		/**
         * Credit Wallet on registration for customer function
         *
         * @param int $user_id
         * @return void
         */
		public function ddwcwm_credit_wallet_on_registration( $user_id ) {

			global $ddwcwm_wallet;

			if ( ! empty( $ddwcwm_wallet[ 'registration_credit' ] ) ) {
				$registration_credit = floatval( $ddwcwm_wallet[ 'registration_credit' ] );

				$user_helper = new DDWCWM_Users_Helper();
				$user_helper->ddwcwm_set_user_wallet_balance( $registration_credit, $user_id );

				$data = [
					'type'      => 'credit',
					'amount'    => $registration_credit,
					'user_id'   => $user_id,
					'note'      => '',
					'date'      => current_time( 'Y-m-d H:i:s' ),
					'reference' => 'registration_credit',
				];

				$transaction_helper = new DDWCWM_Transactions_Helper();

				$transaction_helper->ddwcwm_save_transaction( $data );

				$email_message = [];

				$email_message[] = sprintf( esc_html__( 'You\'ve been credited with amount %s in your wallet as registration credit.', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $registration_credit ) ) );

				$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $registration_credit ) ) );

				$sender_email = get_userdata( $user_id )->user_email;

				$email_data = [
					'email'   => $sender_email,
					'message' => $email_message
				];

				do_action( 'ddwcwm_mail', $email_data );
			}

			if ( ! empty( $_POST[ 'ddwcwm_referral_username' ] ) ) {
				$referred_by_user = get_user_by( 'login', $_POST[ 'ddwcwm_referral_username' ] );

				update_user_meta( $user_id, '_ddwcwm_referred_by', $referred_by_user->ID );

				if ( ! empty( $ddwcwm_wallet[ 'referral_earning_amount' ] ) ) {
					$referral_earning_amount = floatval( $ddwcwm_wallet[ 'referral_earning_amount' ] );

					$user_helper = new DDWCWM_Users_Helper();

					$new_user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $user_id );
					$new_user_wallet_balance += $referral_earning_amount;
					$user_helper->ddwcwm_set_user_wallet_balance( $new_user_wallet_balance, $user_id );

					$data = [
						'type'      => 'credit',
						'amount'    => $referral_earning_amount,
						'user_id'   => $user_id,
						'note'      => sprintf( esc_html__( 'Credited on successful referral by %s <%s>', 'ddwc-wallet-management' ), $referred_by_user->display_name, $referred_by_user->user_email ),
						'date'      => current_time( 'Y-m-d H:i:s' ),
						'reference' => 'referral_credit',
					];

					$transaction_helper = new DDWCWM_Transactions_Helper();

					$transaction_helper->ddwcwm_save_transaction( $data );

					$email_message = [];

					$email_message[] = sprintf( esc_html__( 'You\'ve been credited with amount %s in your wallet as referral credit. You have referred by %s <%s>', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $referral_earning_amount ) ), $referred_by_user->display_name, $referred_by_user->user_email  );

					$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $new_user_wallet_balance ) ) );

					$user_data = get_userdata( $user_id );

					$email_data = [
						'email'   => $user_data->user_email,
						'message' => $email_message
					];

					do_action( 'ddwcwm_mail', $email_data );

					$referral_user_wallet_balance = $user_helper->ddwcwm_get_user_wallet_balance( $referred_by_user->ID );
					$referral_user_wallet_balance += $referral_earning_amount;
					$user_helper->ddwcwm_set_user_wallet_balance( $referral_user_wallet_balance, $referred_by_user->ID );

					$data = [
						'type'      => 'credit',
						'amount'    => $referral_earning_amount,
						'user_id'   => $referred_by_user->ID,
						'note'      => sprintf( esc_html__( 'Credited on successful referral to %s <%s>', 'ddwc-wallet-management' ), $user_data->display_name, $user_data->user_email ),
						'date'      => current_time( 'Y-m-d H:i:s' ),
						'reference' => 'referral_credit',
					];

					$transaction_helper = new DDWCWM_Transactions_Helper();

					$transaction_helper->ddwcwm_save_transaction( $data );

					$email_message = [];

					$email_message[] = sprintf( esc_html__( 'You\'ve been credited with amount %s in your wallet as referral credit. You have referred to %s <%s>', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $referral_earning_amount ) ), $user_data->display_name, $user_data->user_email  );

					$email_message[] = sprintf( esc_html__( 'Available Balance: %s', 'ddwc-wallet-management' ), wc_price( apply_filters( 'ddwcwm_modify_amount_to_multi_currency', $referral_user_wallet_balance ) ) );

					$email_data = [
						'email'   => $referred_by_user->user_email,
						'message' => $email_message
					];

					do_action( 'ddwcwm_mail', $email_data );
				}
			}

		}

	}
}
