<?php
/**
 * Transactions List Template
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Templates\Admin\Transactions;

use DDWCWalletManagement\Helper\Transactions\DDWCWM_Transactions_Helper;
use DDWCWalletManagement\Helper\Error\DDWCWM_Error_Helper;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Transactions_List_Template' ) ) {
	/**
	 * Transactions list class
	 */
	class DDWCWM_Transactions_List_Template extends \WP_List_table {

		/**
		 * Error Helper Trait
		 */
		use DDWCWM_Error_Helper;

        /**
		 * Transaction Helper Variable
		 *
		 * @var object
		 */
		protected $transaction_helper;

		/**
		 * Class constructor
		 */
		public function __construct() {
			$this->transaction_helper = new DDWCWM_Transactions_Helper();

			parent::__construct( [
				'singular' => esc_html__( 'Transaction List', 'ddwc-wallet-management' ),
				'plural'   => esc_html__( 'Transactions List', 'ddwc-wallet-management' ),
				'ajax'     => false,
			] );
		}

		/**
		 * Prepare Items
		 *
		 * @return void
		 */
		public function prepare_items() {
			$this->_column_headers = $this->get_column_info();

			$request_scheme = is_ssl() ? 'https' : 'http';
			$current_url    = "$request_scheme://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

            if ( strpos( $current_url, '_wp_http_referer' ) !== false ) {
                $new_url = remove_query_arg( [ '_wp_http_referer', '_wpnonce' ], stripslashes( $current_url ) );
                wp_safe_redirect( $new_url );
                exit();
            }

			$this->process_bulk_action();
			$this->process_row_action();

			$per_page     = $this->get_items_per_page( 'transactions_per_page', 20 );
			$current_page = $this->get_pagenum();
			$total_items  = $this->transaction_helper->ddwcwm_get_all_transactions_count();

			$this->set_pagination_args( [
				'total_items' => $total_items,
				'per_page'    => $per_page,
			] );

			$data = self::ddwcwm_get_transactions( $per_page, $current_page );

			usort( $data, [ $this, 'usort_reorder' ] );

			$this->items = $data;
		}

		/**
		 * Process bulk actions
		 *
		 * @return void
		 */
		public function process_bulk_action() {

			if ( ! empty( $_GET[ 'ddwcwm_transactions_list_nonce' ] ) && wp_unslash( $_GET[ 'ddwcwm_transactions_list_nonce' ] ) ) { // WPCS: CSRF ok. // WPCS: input var ok. // WPCS: sanitization ok.
				$nonce = filter_input( INPUT_GET, 'ddwcwm_transactions_list_nonce', FILTER_SANITIZE_STRING );
				if ( wp_verify_nonce( $nonce, 'ddwcwm_transactions_list_nonce_action' ) ) {
					if ( 'delete' === $this->current_action() ) {
						if ( ! empty( $_GET[ 'ddwcwm-transaction-id' ] ) ) { // WPCS: input var ok.

							if ( is_array( $_GET[ 'ddwcwm-transaction-id' ] ) ) { // WPCS: input var ok.
								$transaction_ids = array_map( 'sanitize_text_field', wp_unslash( $_GET[ 'ddwcwm-transaction-id' ] ) ); // WPCS: input var ok.

								foreach ( $transaction_ids as $transaction_id ) {
									$this->transaction_helper->ddwcwm_delete_transaction( $transaction_id );
								}

								$message = sprintf( esc_html__( '%s Transaction(s) deleted successfully.', 'ddwc-wallet-management' ), count( $transaction_ids ) );
								$this->ddwcwm_print_notification( $message, 'success' );

							}
						} else {
							$message = esc_html__( 'Select transactions(s) to delete.', 'ddwc-wallet-management' );
							$this->ddwcwm_print_notification( $message, 'error' );
						}
					}
				} else {
					$message = esc_html__( 'Invalid nonce. Security check failed!!!', 'ddwc-wallet-management' ) . implode( ',', $error_ids );
					$this->ddwcwm_print_notification( $message, 'error' );
				}
			}
		}

		/**
		 * Process row actions
		 *
		 * @return void
		 */
		public function process_row_action() {

			if ( ! empty( $_GET[ 'ddwcwm_transactions_list_nonce' ] ) && wp_unslash( $_GET[ 'ddwcwm_transactions_list_nonce' ] ) ) { // WPCS: CSRF ok. // WPCS: input var ok. // WPCS: sanitization ok.
				$nonce = filter_input( INPUT_GET, 'ddwcwm_transactions_list_nonce', FILTER_SANITIZE_STRING );
				if ( wp_verify_nonce( $nonce, 'ddwcwm_transactions_list_nonce_action' ) ) {
					if ( 'delete' === $this->current_action() ) {
						if ( ! empty( $_GET[ 'ddwcwm-transaction-id' ] ) && ! is_array( $_GET[ 'ddwcwm-transaction-id' ] ) ) { // WPCS: input var ok.
							$transaction_id = intval( wp_unslash( $_GET[ 'ddwcwm-transaction-id' ] ) ); // WPCS: input var ok.
							$this->transaction_helper->ddwcwm_delete_transaction( $transaction_id );

							$message = esc_html__( 'Transaction deleted successfully.', 'ddwc-wallet-management' );
							$this->ddwcwm_print_notification( $message, 'success' );

						}
					}
				} else {
					$message = esc_html__( 'Invalid nonce. Security check failed!!!', 'ddwc-wallet-management' ) . implode( ',', $error_ids );
					$this->ddwcwm_print_notification( $message, 'error' );
				}
			}
		}

		/**
		 * Usort
		 *
		 * @param int $first First value.
		 * @param int $second Second value.
		 * @return $result
		 */
		public function usort_reorder( $first, $second ){

			$orderby = ! empty( $_GET[ 'orderby' ] ) ? $_GET[ 'orderby' ] : 'id';
			$order = ! empty( $_GET[ 'order' ] ) ? $_GET[ 'order' ] : 'desc';

			$result = strnatcmp( $first[ $orderby ], $second[ $orderby ] );
			return $order === 'asc' ? $result : -$result;

		}

		/**
		 * Fetch Transactions
		 *
		 * @param int $per_page Per Page.
		 * @param int $current_page Page.
		 * @return array $transactions
		 */
		public function ddwcwm_get_transactions( $per_page, $current_page = 1 ) {

            $data = [];

			$offset = ( $current_page - 1 ) * $per_page;

            $transactions = $this->transaction_helper->ddwcwm_get_all_transactions( $per_page, $offset );

            if ( ! empty( $transactions ) ) {

				foreach ( $transactions as $key => $transaction ) {

					$transaction_id        = $transaction[ 'id' ];
					$customer_id           = $transaction[ 'user_id' ];
					$order_id              = $transaction[ 'order_id' ];
					$order_currency        = get_post_meta( $order_id, '_order_currency', true );
					$order_currency        = apply_filters( 'ddwcwm_modify_transaction_currency', $order_currency, $order_id, $transaction );
					$customer              = get_user_by( 'ID', $customer_id );
					$transaction_type      = $transaction[ 'type' ];
					$transaction_reference = $transaction[ 'reference' ];

					ob_start();
					if ( $transaction_type == 'credit' || $transaction_reference == 'withdraw_cancelled' ) {
						?>
						<span class="ddwcwm-credit">+<?php echo wc_price( $transaction[ 'amount' ], [ 'currency' => $order_currency ] ) ?></span>
						<?php
					} else if ( $transaction_type == 'transfer' || $transaction_reference == 'withdraw_completed' ) {
						?>
						<span class="ddwcwm-complete"><?php echo wc_price( $transaction[ 'amount' ], [ 'currency' => $order_currency ] ) ?></span>
						<?php
					} else if ( $transaction_type == 'debit' || $transaction_type == 'withdraw' ) {
						?>
						<span class="ddwcwm-debit">-<?php echo wc_price( $transaction[ 'amount' ], [ 'currency' => $order_currency ] ) ?></span>
						<?php
					}
					$amount = ob_get_contents();
					ob_end_clean();

					if ( ! empty( $customer ) && is_object( $customer ) ) {

						$email = $customer->user_email . ' (#' . $customer_id . ')';
						$data[] = [
							'id'             => $transaction_id,
							'reference'      => $this->transaction_helper->ddwcwm_get_transactions_translation( $transaction_reference ),
							'customer_email' => '<a href=' . esc_url( admin_url( 'user-edit.php?user_id=' . $customer_id ) ) . '>' . esc_html( $email ) . '</a>',
							'amount'         => $amount,
							'type'           => $this->transaction_helper->ddwcwm_get_transactions_translation( $transaction_type ),
							'date'           => date_i18n( 'F d, Y g:i:s A', strtotime( $transaction[ 'date' ] ) ),
                        ];
					}

				}
			}

			return apply_filters( 'ddwcwm_transactions_list_data', $data );
		}

		/**
		 *  No items
		 *
		 * @return void
		 */
		public function no_items() {
			esc_html_e( 'No transactions avaliable.', 'ddwc-wallet-management' );
		}

		/**
		 * Hidden Columns
		 *
		 * @return array
		 */
		public function get_hidden_columns() {
			return [];
		}

		/**
		 *  Associative array of columns
		 *
		 * @return array
		 */
		public function get_columns() {
			$columns = [
				'cb'             => '<input type="checkbox" />',
				'id'             => esc_html__( 'ID', 'ddwc-wallet-management' ),
				'reference'      => esc_html__( 'Reference', 'ddwc-wallet-management' ),
				'customer_email' => esc_html__( 'Customer Email', 'ddwc-wallet-management' ),
				'amount'         => esc_html__( 'Amount', 'ddwc-wallet-management' ),
				'type'           => esc_html__( 'Type', 'ddwc-wallet-management' ),
				'date'           => esc_html__( 'Date', 'ddwc-wallet-management' ),
			];

			return apply_filters( 'ddwcwm_transactions_list_columns', $columns );
		}

		/**
		 * Render a column when no column specific method exists.
		 *
		 * @param array  $item Items.
		 * @param string $column_name Name.
		 *
		 * @return mixed
		 */
		public function column_default( $item, $column_name ) {
			if ( array_key_exists( $column_name, $item ) ) {
				return $item[ $column_name ];
			}

			return print_r( $item, true );
		}

		/**
		 * Columns to make sortable.
		 *
		 * @return array
		 */
		public function get_sortable_columns() {
			$sortable_columns = [
				'id'             => [ 'id', true ],
				'reference'      => [ 'reference', true ],
				'customer_email' => [ 'customer_email', true ],
				'amount'         => [ 'amount', true ],
			];

			return apply_filters( 'ddwcwm_transactions_list_sortable_columns', $sortable_columns );
		}

		/**
		 * Column actions
		 *
		 * @param array $item Items.
		 * @return array $actions
		 */
		public function column_id( $item ) {

			$actions = [
				'view'   => sprintf( '<a href="%s">%s</a>', esc_url( admin_url( 'admin.php?page=' . $_REQUEST[ 'page' ] . '&action=view&transaction_id=' . $item[ 'id' ] ) ), esc_html__( 'View', 'ddwc-wallet-management' ) ),
				'delete' => sprintf( '<a href="%s">%s</a>', wp_nonce_url( 'admin.php?page=' . $_REQUEST[ 'page' ] . '&action=delete&ddwcwm-transaction-id=' . esc_attr( $item[ 'id' ] ), 'ddwcwm_transactions_list_nonce_action', 'ddwcwm_transactions_list_nonce' ), esc_html__( 'Delete', 'ddwc-wallet-management' ) ),
			];
			return sprintf( '%1$s %2$s', $item[ 'id' ], $this->row_actions( apply_filters( 'ddwcwm_transactions_list_line_actions', $actions ) ) );
		}

		/**
		 * Render the bulk edit checkbox
		 *
		 * @param array $item Item.
		 * @return string
		 */
		public function column_cb( $item ) {
			return sprintf( '<input type="checkbox" name="ddwcwm-transaction-id[]" value="%d" />', esc_attr( $item[ 'id' ] ) );
		}

		/**
         * Bulk actions on list.
		 * 
		 * @return array
         */
        public function get_bulk_actions() {
            return apply_filters( 'ddwcwm_modify_bulk_actions_in_transactions', [
                'delete' => esc_html__( 'Delete', 'ddwc-wallet-management' ),
			] );
        }

        /**
		 * Transactions List Filters
		 *
		 * @param string $which Position of filter.
		 */
		public function extra_tablenav( $which ) {
			$all_transaction_types = [
				''         => esc_html__( 'Transaction Type', 'ddwc-wallet-management' ),
				'credit'   => esc_html__( 'Credit', 'ddwc-wallet-management' ),
				'debit'    => esc_html__( 'Debit', 'ddwc-wallet-management' ),
				'transfer' => esc_html__( 'Transfer', 'ddwc-wallet-management' ),
				'withdraw' => esc_html__( 'Withdrawal', 'ddwc-wallet-management' ),
			];

			$all_transaction_types = apply_filters( 'ddwcwm_modify_transaction_types_for_filter' , $all_transaction_types );

			if ( 'top' === $which ) {
                $transaction_type      = ! empty( $_GET[ 'ddwcwm-transaction-type' ] ) ? $_GET[ 'ddwcwm-transaction-type' ] : '';
                $transaction_from_date = ! empty( $_GET[ 'ddwcwm-transaction-from-date' ] ) ? $_GET[ 'ddwcwm-transaction-from-date' ] : '';
                $transaction_to_date   = ! empty( $_GET[ 'ddwcwm-transaction-to-date' ] ) ? $_GET[ 'ddwcwm-transaction-to-date' ] : '';
				?>
				<div class="alignleft actions bulkactions ddwcwm-transactions-bulk-actions">
					<select name="ddwcwm-transaction-type" class="ddwcwm-transaction-type">
						<?php
						if ( ! empty( $all_transaction_types ) ) {
							foreach ( $all_transaction_types as $key => $value ) {

								$selected = $key == $transaction_type ? 'selected="selected"' : '';
								?>
								<option value="<?php echo esc_attr( $key ); ?>" <?php echo esc_attr( $selected ); ?>><?php echo esc_html( $value ); ?></option>
								<?php
							}
						}
						?>

					</select>

					<label for="ddwcwm-transaction-from-date"><?php esc_html_e( 'From:', 'ddwc-wallet-management' ); ?></label>
					<input type="text" value="<?php echo esc_attr( $transaction_from_date ); ?>" name="ddwcwm-transaction-from-date" id="ddwcwm-transaction-from-date" placeholder="yyyy-mm-dd" autocomplete="off" />

					<label for="ddwcwm-transaction-to-date"><?php esc_html_e( 'To:', 'ddwc-wallet-management' ); ?></label>
					<input type="text" value="<?php echo esc_attr( $transaction_to_date ); ?>" name="ddwcwm-transaction-to-date" id="ddwcwm-transaction-to-date" placeholder="yyyy-mm-dd" autocomplete="off" />

					<label for="ddwcwm-users"><?php esc_html_e( 'Customer:', 'ddwc-wallet-management' ); ?></label>
					<select class="ddwcwm-users" name="customer-id" id="ddwcwm-users" title="<?php esc_html_e( 'Customer', 'ddwc-wallet-management' ); ?>" data-placeholder="<?php esc_attr_e( 'Select Customer', 'ddwc-wallet-management' ); ?>">
						<?php
						if ( ! empty( $_GET[ 'customer-id' ] ) ) {
							$customer_id = $_GET[ 'customer-id' ];
							$customer_data = get_userdata( $customer_id );
							$customer_option_value = "(#{$customer_id}) {$customer_data->user_login} <{$customer_data->user_email}>";
							?>
							<option value="<?php echo esc_attr( $customer_id ); ?>"><?php echo esc_html( $customer_option_value ); ?></option>
							<?php
						}
						?>
					</select>

					<input type="submit" value="<?php esc_html_e( 'Filter', 'ddwc-wallet-management' ); ?>" name="transaction" class="button" />
				</div>
				<?php
			}
		}

	}
}
