<?php
/**
 * Withdrawals List Template
 *
 * @package WooCommerce Wallet Management
 * @version 1.0.0
 */

namespace DDWCWalletManagement\Templates\Admin\Withdrawals;

use DDWCWalletManagement\Helper\Error\DDWCWM_Error_Helper;
use DDWCWalletManagement\Helper\Users\DDWCWM_Users_Helper;

defined( 'ABSPATH' ) || exit();

if ( ! class_exists( 'DDWCWM_Withdrawals_List_Template' ) ) {

	/**
	 * Withdrawals list class
	 */
	class DDWCWM_Withdrawals_List_Template extends \WP_List_table {

		/**
		 * Error Helper Trait
		 */
		use DDWCWM_Error_Helper;

		/**
		 * Users Helper Variable
		 *
		 * @var object
		 */
		protected $user_helper;

		/**
		 * Class constructor
		 */
		public function __construct() {
            $this->user_helper = new DDWCWM_Users_Helper();

			parent::__construct( [
				'singular' => esc_html__( 'Withdrawal Request List', 'ddwc-wallet-management' ),
				'plural'   => esc_html__( 'Withdrawal Requests List', 'ddwc-wallet-management' ),
				'ajax'     => false,
			] );
		}

		/**
		 * Prepare Items
		 *
		 * @return void
		 */
		public function prepare_items() {
			$this->_column_headers = $this->get_column_info();

			$request_scheme = is_ssl() ? 'https' : 'http';
			$current_url    = "$request_scheme://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

            if ( strpos( $current_url, '_wp_http_referer' ) !== false ) {
                $new_url = remove_query_arg( [ '_wp_http_referer', '_wpnonce' ], stripslashes( $current_url ) );
                wp_safe_redirect( $new_url );
                exit();
            }

            $search = ! empty( $_GET[ 's' ] ) ? sanitize_text_field( wp_unslash( $_GET[ 's' ] ) ) : '';

			$per_page     = $this->get_items_per_page( 'withdrawals_per_page', 20 );
			$current_page = $this->get_pagenum();
			$total_items  = self::ddwcwm_users_count( $search );

			$this->set_pagination_args( [
				'total_items' => $total_items,
				'per_page'    => $per_page,
			] );

			$data = self::ddwcwm_get_users( $per_page, $current_page, $search );

			usort( $data, [ $this, 'usort_reorder' ] );

			$this->items = $data;
		}

		/**
		 * Usort
		 *
		 * @param int $first First value.
		 * @param int $second Second value.
		 * @return $result
		 */
		public function usort_reorder( $first, $second ) {
			$orderby = ! empty( $_GET[ 'orderby' ] ) ? $_GET[ 'orderby' ] : 'id';
			$order   = ! empty( $_GET[ 'order' ] ) ? $_GET[ 'order' ] : 'asc';
			$result  = strnatcmp( $first[$orderby], $second[$orderby] );

			return 'asc' === $order ? $result : -$result;
		}

		/**
		 * Record count
		 *
		 * @param string $search Search.
		 * @return $count
		 */
		public function ddwcwm_users_count( $search = '' ) {

			$user_query = new \WP_User_Query( [
				'search'         => '*' . esc_attr( $search ) . '*',
				'search_columns' => [ 'user_nicename', 'ID', 'user_login', 'user_email' ],
				'meta_query' => [
					[
						'key' => '_ddwcwm_wallet_withdraw_requests',
						'compare' => 'EXISTS'
					]
				]
			] );

            return $user_query->get_total();

		}

		/**
		 * Fetch Users
		 *
		 * @param int $per_page Per Page.
		 * @param int $current_page Page.
		 * @param string $search Search.
		 * @return array $users
		 */
		public function ddwcwm_get_users( $per_page, $current_page = 1, $search = '' ) {
            $data = [];
            $off  = ( $current_page - 1 ) * $per_page;

			$query = new \WP_User_Query( [
				'number'         => $per_page,
				'offset'         => $off,
				'order'          => 'DESC',
				'orderby'        => 'ID',
				'search'         => '*' . esc_attr( $search ) . '*',
				'search_columns' => [ 'user_nicename', 'ID', 'user_login', 'user_email' ],
				'fields'         => [ 'ID', 'user_login', 'user_email' ],
				'meta_query' => [
					[
						'key'     => '_ddwcwm_wallet_withdraw_requests',
						'compare' => 'EXISTS'
					]
				]
			] );

            $users = $query->get_results();

            if ( ! empty( $users ) ) {
                foreach ( $users as $user ) {
					$user_id           = $user->ID;
					$balance           = $this->user_helper->ddwcwm_get_user_wallet_balance( $user_id );
					$withdraw_requests = $this->user_helper->ddwcwm_get_user_withdraw_requests( $user_id );

					ob_start();
					?>
					<button class="button ddwcwm-view-withdraw-requests" data-user-id="<?php echo esc_attr( $user_id ); ?>" title="<?php esc_attr_e( 'View Requests', 'ddwc-wallet-management' ); ?>"><?php esc_html_e( 'View Requests', 'ddwc-wallet-management' ); ?></button>
					<div class="ddwcwm-wallet-withdraw-requests-popup" id="ddwcwm-wallet-withdraw-requests-<?php echo esc_attr( $user_id ); ?>">
						<div class="ddwcwm-wallet-withdraw-requests-popup-content">
							<h3><?php esc_html_e( 'Withdrawal Requests', 'ddwc-wallet-management' ); ?></h3>
							<span class="ddwcwm-close-popup"></span>
							<div class="ddwcwm-message ddwcwm-hide"><p></p></div>
							<table class="striped widefat fixed">
								<thead>
									<tr>
										<th><?php esc_html_e( 'Withdraw Amount', 'ddwc-wallet-management' ); ?></th>
										<th><?php esc_html_e( 'Charges', 'ddwc-wallet-management' ); ?></th>
										<th><?php esc_html_e( 'Amount deducting charges', 'ddwc-wallet-management' ); ?></th>
										<th><?php esc_html_e( 'Note', 'ddwc-wallet-management' ); ?></th>
										<th><?php esc_html_e( 'Requested on', 'ddwc-wallet-management' ); ?></th>
										<th><?php esc_html_e( 'Action', 'ddwc-wallet-management' ); ?></th>
									</tr>
								</thead>
								<tbody>
									<?php
									if ( ! empty( $withdraw_requests ) ) {
										foreach( $withdraw_requests as $key => $withdraw_request ) {
											?>
											<tr data-withdraw-request-id="<?php echo esc_attr( $key ); ?>">
												<td><?php echo wc_price( $withdraw_request[ 'amount_before_charges' ] ); ?></td>
												<td><?php echo wc_price( $withdraw_request[ 'charges' ] ); ?></td>
												<td><?php echo wc_price( $withdraw_request[ 'amount_after_charges' ] ); ?></td>
												<td><?php echo esc_html( $withdraw_request[ 'note' ] ); ?></td>
												<td><?php echo date_i18n( 'F d, Y g:i:s A', strtotime( $withdraw_request[ 'date' ] ) ); ?></td>
												<td>
													<button class="button ddwcwm-withdraw-request-action" data-withdraw-request-action="mark_as_paid" data-withdraw-request-id="<?php echo esc_attr( $key ); ?>" data-user-id="<?php echo esc_attr( $user_id ); ?>" title="<?php esc_attr_e( 'Mark as Paid', 'ddwc-wallet-management' ); ?>"><?php esc_html_e( 'Mark as Paid', 'ddwc-wallet-management' ); ?></button>

													<button class="button ddwcwm-withdraw-request-action" data-withdraw-request-action="cancel" data-withdraw-request-id="<?php echo esc_attr( $key ); ?>" data-user-id="<?php echo esc_attr( $user_id ); ?>" title="<?php esc_attr_e( 'Cancel', 'ddwc-wallet-management' ); ?>"><?php esc_html_e( 'Cancel', 'ddwc-wallet-management' ); ?></button>
												</td>
											</tr>
											<?php
										}
									}
									?>
								</tbody>
							</table>
						</div>
					</div>
					<?php
					$actions = ob_get_contents();
					ob_end_clean();

                    $data[] = [
                        'id'       => $user_id,
                        'username' => $user->user_login,
                        'email'    => $user->user_email,
                        'balance'  => wc_price( $balance ),
                        'actions'  => $actions
					];
                }
            }

			return apply_filters( 'ddwcwm_withdrawals_list_data', $data );
		}

		/**
		 *  No items
		 *
		 * @return void
		 */
		public function no_items() {
			esc_html_e( 'No Withdrawal requests avaliable.', 'ddwc-wallet-management' );
		}

		/**
		 * Hidden Columns
		 *
		 * @return array
		 */
		public function get_hidden_columns() {
			return [];
		}

		/**
		 *  Associative array of columns
		 *
		 * @return array
		 */
		public function get_columns() {
			$columns = [
				'id'       => esc_html__( 'ID', 'ddwc-wallet-management' ),
				'username' => esc_html__( 'Username', 'ddwc-wallet-management' ),
				'email'    => esc_html__( 'Email', 'ddwc-wallet-management' ),
				'balance'  => esc_html__( 'Balance', 'ddwc-wallet-management' ),
				'actions'  => esc_html__( 'Actions', 'ddwc-wallet-management' ),
			];

			return apply_filters( 'ddwcwm_withdrawals_list_columns', $columns );
		}

		/**
		 * Render a column when no column specific method exists.
		 *
		 * @param array  $item Items.
		 * @param string $column_name Name.
		 *
		 * @return mixed
		 */
		public function column_default( $item, $column_name ) {
			if ( array_key_exists( $column_name, $item ) ) {
				return $item[ $column_name ];
			}

			return print_r( $item, true );
		}

		/**
		 * Columns to make sortable.
		 *
		 * @return array
		 */
		public function get_sortable_columns() {
			$sortable_columns = [
				'id'       => [ 'id', true ],
				'username' => [ 'username', true ],
				'email'    => [ 'email', true ],
				'balance'  => [ 'balance', true ],
			];

			return apply_filters( 'ddwcwm_withdrawals_list_sortable_columns', $sortable_columns );
		}

	}

}
